@@ -46,11 +46,11 @@ typedef struct _ValaCCodeAssignmentPrivate ValaCCodeAssignmentPrivate;
  * Represents an assignment expression in the C code.
  */
 struct _ValaCCodeAssignment {
-	ValaCCodeExpression parent;
+	ValaCCodeExpression parent_instance;
 	ValaCCodeAssignmentPrivate * priv;
 };
 struct _ValaCCodeAssignmentClass {
-	ValaCCodeExpressionClass parent;
+	ValaCCodeExpressionClass parent_class;
 };
 typedef enum  {
 	VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE,
@@ -68,11 +68,9 @@ typedef enum  {
 
 ValaCCodeAssignment* vala_ccode_assignment_new (ValaCCodeExpression* l, ValaCCodeExpression* r, ValaCCodeAssignmentOperator op);
 ValaCCodeExpression* vala_ccode_assignment_get_left (ValaCCodeAssignment* self);
-void vala_ccode_assignment_set_left (ValaCCodeAssignment* self, ValaCCodeExpression* value);
 ValaCCodeAssignmentOperator vala_ccode_assignment_get_operator (ValaCCodeAssignment* self);
 void vala_ccode_assignment_set_operator (ValaCCodeAssignment* self, ValaCCodeAssignmentOperator value);
 ValaCCodeExpression* vala_ccode_assignment_get_right (ValaCCodeAssignment* self);
-void vala_ccode_assignment_set_right (ValaCCodeAssignment* self, ValaCCodeExpression* value);
 GType vala_ccode_assignment_get_type (void);
 
 G_END_DECLS
