@@ -46,11 +46,11 @@ typedef struct _ValaCCodeBinaryExpressionPrivate ValaCCodeBinaryExpressionPrivat
  * Represents an expression with two operands in C code.
  */
 struct _ValaCCodeBinaryExpression {
-	ValaCCodeExpression parent;
+	ValaCCodeExpression parent_instance;
 	ValaCCodeBinaryExpressionPrivate * priv;
 };
 struct _ValaCCodeBinaryExpressionClass {
-	ValaCCodeExpressionClass parent;
+	ValaCCodeExpressionClass parent_class;
 };
 typedef enum  {
 	VALA_CCODE_BINARY_OPERATOR_PLUS,
