@@ -46,9 +46,9 @@ ValaCCodeConditionalExpression* vala_ccode_conditional_expression_new (ValaCCode
 	GParameter * __params;
 	GParameter * __params_it;
 	ValaCCodeConditionalExpression * self;
-	g_return_val_if_fail (VALA_IS_CCODE_EXPRESSION (cond), NULL);
-	g_return_val_if_fail (VALA_IS_CCODE_EXPRESSION (true_expr), NULL);
-	g_return_val_if_fail (VALA_IS_CCODE_EXPRESSION (false_expr), NULL);
+	g_return_val_if_fail (cond == NULL || VALA_IS_CCODE_EXPRESSION (cond), NULL);
+	g_return_val_if_fail (true_expr == NULL || VALA_IS_CCODE_EXPRESSION (true_expr), NULL);
+	g_return_val_if_fail (false_expr == NULL || VALA_IS_CCODE_EXPRESSION (false_expr), NULL);
 	__params = g_new0 (GParameter, 3);
 	__params_it = __params;
 	(__params_it->name = "condition", g_value_init (&__params_it->value, VALA_TYPE_CCODE_EXPRESSION), g_value_set_object (&__params_it->value, cond), __params_it++);
@@ -67,7 +67,7 @@ ValaCCodeConditionalExpression* vala_ccode_conditional_expression_new (ValaCCode
 static void vala_ccode_conditional_expression_real_write (ValaCCodeNode* base, ValaCCodeWriter* writer) {
 	ValaCCodeConditionalExpression * self;
 	self = VALA_CCODE_CONDITIONAL_EXPRESSION (base);
-	g_return_if_fail (VALA_IS_CCODE_WRITER (writer));
+	g_return_if_fail (writer == NULL || VALA_IS_CCODE_WRITER (writer));
 	vala_ccode_writer_write_string (writer, "(");
 	vala_ccode_node_write (VALA_CCODE_NODE (vala_ccode_conditional_expression_get_condition (self)), writer);
 	vala_ccode_writer_write_string (writer, " ? ");
@@ -85,10 +85,12 @@ ValaCCodeExpression* vala_ccode_conditional_expression_get_condition (ValaCCodeC
 
 
 void vala_ccode_conditional_expression_set_condition (ValaCCodeConditionalExpression* self, ValaCCodeExpression* value) {
+	ValaCCodeExpression* _tmp2;
 	ValaCCodeExpression* _tmp1;
 	g_return_if_fail (VALA_IS_CCODE_CONDITIONAL_EXPRESSION (self));
+	_tmp2 = NULL;
 	_tmp1 = NULL;
-	self->priv->_condition = (_tmp1 = g_object_ref (value), (self->priv->_condition == NULL ? NULL : (self->priv->_condition = (g_object_unref (self->priv->_condition), NULL))), _tmp1);
+	self->priv->_condition = (_tmp2 = (_tmp1 = value, (_tmp1 == NULL ? NULL : g_object_ref (_tmp1))), (self->priv->_condition == NULL ? NULL : (self->priv->_condition = (g_object_unref (self->priv->_condition), NULL))), _tmp2);
 }
 
 
@@ -99,10 +101,12 @@ ValaCCodeExpression* vala_ccode_conditional_expression_get_true_expression (Vala
 
 
 void vala_ccode_conditional_expression_set_true_expression (ValaCCodeConditionalExpression* self, ValaCCodeExpression* value) {
+	ValaCCodeExpression* _tmp2;
 	ValaCCodeExpression* _tmp1;
 	g_return_if_fail (VALA_IS_CCODE_CONDITIONAL_EXPRESSION (self));
+	_tmp2 = NULL;
 	_tmp1 = NULL;
-	self->priv->_true_expression = (_tmp1 = g_object_ref (value), (self->priv->_true_expression == NULL ? NULL : (self->priv->_true_expression = (g_object_unref (self->priv->_true_expression), NULL))), _tmp1);
+	self->priv->_true_expression = (_tmp2 = (_tmp1 = value, (_tmp1 == NULL ? NULL : g_object_ref (_tmp1))), (self->priv->_true_expression == NULL ? NULL : (self->priv->_true_expression = (g_object_unref (self->priv->_true_expression), NULL))), _tmp2);
 }
 
 
@@ -113,10 +117,12 @@ ValaCCodeExpression* vala_ccode_conditional_expression_get_false_expression (Val
 
 
 void vala_ccode_conditional_expression_set_false_expression (ValaCCodeConditionalExpression* self, ValaCCodeExpression* value) {
+	ValaCCodeExpression* _tmp2;
 	ValaCCodeExpression* _tmp1;
 	g_return_if_fail (VALA_IS_CCODE_CONDITIONAL_EXPRESSION (self));
+	_tmp2 = NULL;
 	_tmp1 = NULL;
-	self->priv->_false_expression = (_tmp1 = g_object_ref (value), (self->priv->_false_expression == NULL ? NULL : (self->priv->_false_expression = (g_object_unref (self->priv->_false_expression), NULL))), _tmp1);
+	self->priv->_false_expression = (_tmp2 = (_tmp1 = value, (_tmp1 == NULL ? NULL : g_object_ref (_tmp1))), (self->priv->_false_expression == NULL ? NULL : (self->priv->_false_expression = (g_object_unref (self->priv->_false_expression), NULL))), _tmp2);
 }
 
 
