@@ -46,7 +46,6 @@ static void vala_ccode_declaration_dispose (GObject * obj);
 
 ValaCCodeDeclaration* vala_ccode_declaration_new (const char* type_name) {
 	ValaCCodeDeclaration * self;
-	g_return_val_if_fail (type_name != NULL, NULL);
 	self = g_object_newv (VALA_TYPE_CCODE_DECLARATION, 0, NULL);
 	vala_ccode_declaration_set_type_name (self, type_name);
 	return self;
@@ -60,7 +59,7 @@ ValaCCodeDeclaration* vala_ccode_declaration_new (const char* type_name) {
  */
 void vala_ccode_declaration_add_declarator (ValaCCodeDeclaration* self, ValaCCodeDeclarator* decl) {
 	g_return_if_fail (VALA_IS_CCODE_DECLARATION (self));
-	g_return_if_fail (VALA_IS_CCODE_DECLARATOR (decl));
+	g_return_if_fail (decl == NULL || VALA_IS_CCODE_DECLARATOR (decl));
 	gee_collection_add (GEE_COLLECTION (self->priv->declarators), decl);
 }
 
@@ -68,7 +67,7 @@ void vala_ccode_declaration_add_declarator (ValaCCodeDeclaration* self, ValaCCod
 static void vala_ccode_declaration_real_write (ValaCCodeNode* base, ValaCCodeWriter* writer) {
 	ValaCCodeDeclaration * self;
 	self = VALA_CCODE_DECLARATION (base);
-	g_return_if_fail (VALA_IS_CCODE_WRITER (writer));
+	g_return_if_fail (writer == NULL || VALA_IS_CCODE_WRITER (writer));
 	if ((vala_ccode_declaration_get_modifiers (self) & (VALA_CCODE_MODIFIERS_STATIC | VALA_CCODE_MODIFIERS_EXTERN)) != 0) {
 		gboolean first;
 		/* combined declaration and initialization for static and extern variables*/
@@ -125,7 +124,7 @@ static void vala_ccode_declaration_real_write_declaration (ValaCCodeNode* base,
 	ValaCCodeDeclaration * self;
 	gboolean first;
 	self = VALA_CCODE_DECLARATION (base);
-	g_return_if_fail (VALA_IS_CCODE_WRITER (writer));
+	g_return_if_fail (writer == NULL || VALA_IS_CCODE_WRITER (writer));
 	if ((vala_ccode_declaration_get_modifiers (self) & (VALA_CCODE_MODIFIERS_STATIC | VALA_CCODE_MODIFIERS_EXTERN)) != 0) {
 		return;
 	}
@@ -169,10 +168,12 @@ char* vala_ccode_declaration_get_type_name (ValaCCodeDeclaration* self) {
 
 
 void vala_ccode_declaration_set_type_name (ValaCCodeDeclaration* self, const char* value) {
-	char* _tmp1;
+	char* _tmp2;
+	const char* _tmp1;
 	g_return_if_fail (VALA_IS_CCODE_DECLARATION (self));
+	_tmp2 = NULL;
 	_tmp1 = NULL;
-	self->priv->_type_name = (_tmp1 = g_strdup (value), (self->priv->_type_name = (g_free (self->priv->_type_name), NULL)), _tmp1);
+	self->priv->_type_name = (_tmp2 = (_tmp1 = value, (_tmp1 == NULL ? NULL : g_strdup (_tmp1))), (self->priv->_type_name = (g_free (self->priv->_type_name), NULL)), _tmp2);
 }
 
 
