@@ -5,7 +5,7 @@
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * version 2.1 of the License, or (at your option) any later version.
 
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -20,8 +20,8 @@
  * 	Jürg Billeter <j@bitron.ch>
  */
 
-#include "valaccodeincludedirective.h"
 #include <ccode/valaccodeincludedirective.h>
+#include <ccode/valaccodelinedirective.h>
 
 struct _ValaCCodeIncludeDirectivePrivate {
 	char* _filename;
@@ -38,12 +38,10 @@ static gpointer vala_ccode_include_directive_parent_class = NULL;
 static void vala_ccode_include_directive_dispose (GObject * obj);
 
 
-ValaCCodeIncludeDirective* vala_ccode_include_directive_new (const char* _filename, gboolean _local)
-{
+ValaCCodeIncludeDirective* vala_ccode_include_directive_new (const char* _filename, gboolean _local) {
 	GParameter * __params;
 	GParameter * __params_it;
 	ValaCCodeIncludeDirective * self;
-	g_return_val_if_fail (_filename != NULL, NULL);
 	__params = g_new0 (GParameter, 1);
 	__params_it = __params;
 	(__params_it->name = "filename", g_value_init (&__params_it->value, G_TYPE_STRING), g_value_set_string (&__params_it->value, _filename), __params_it++);
@@ -58,12 +56,11 @@ ValaCCodeIncludeDirective* vala_ccode_include_directive_new (const char* _filena
 }
 
 
-static void vala_ccode_include_directive_real_write (ValaCCodeNode* base, ValaCCodeWriter* writer)
-{
+static void vala_ccode_include_directive_real_write (ValaCCodeNode* base, ValaCCodeWriter* writer) {
 	ValaCCodeIncludeDirective * self;
-	self = ((ValaCCodeIncludeDirective*) base);
-	g_return_if_fail (VALA_IS_CCODE_WRITER (writer));
-	vala_ccode_writer_write_indent (writer);
+	self = VALA_CCODE_INCLUDE_DIRECTIVE (base);
+	g_return_if_fail (writer == NULL || VALA_IS_CCODE_WRITER (writer));
+	vala_ccode_writer_write_indent (writer, NULL);
 	vala_ccode_writer_write_string (writer, "#include ");
 	if (vala_ccode_include_directive_get_local (self)) {
 		vala_ccode_writer_write_string (writer, "\"");
@@ -80,38 +77,35 @@ static void vala_ccode_include_directive_real_write (ValaCCodeNode* base, ValaCC
 }
 
 
-char* vala_ccode_include_directive_get_filename (ValaCCodeIncludeDirective* self)
-{
+char* vala_ccode_include_directive_get_filename (ValaCCodeIncludeDirective* self) {
 	g_return_val_if_fail (VALA_IS_CCODE_INCLUDE_DIRECTIVE (self), NULL);
 	return self->priv->_filename;
 }
 
 
-void vala_ccode_include_directive_set_filename (ValaCCodeIncludeDirective* self, const char* value)
-{
-	char* _tmp1;
+void vala_ccode_include_directive_set_filename (ValaCCodeIncludeDirective* self, const char* value) {
+	char* _tmp2;
+	const char* _tmp1;
 	g_return_if_fail (VALA_IS_CCODE_INCLUDE_DIRECTIVE (self));
+	_tmp2 = NULL;
 	_tmp1 = NULL;
-	self->priv->_filename = (_tmp1 = g_strdup (value), (self->priv->_filename = (g_free (self->priv->_filename), NULL)), _tmp1);
+	self->priv->_filename = (_tmp2 = (_tmp1 = value, (_tmp1 == NULL ? NULL : g_strdup (_tmp1))), (self->priv->_filename = (g_free (self->priv->_filename), NULL)), _tmp2);
 }
 
 
-gboolean vala_ccode_include_directive_get_local (ValaCCodeIncludeDirective* self)
-{
+gboolean vala_ccode_include_directive_get_local (ValaCCodeIncludeDirective* self) {
 	g_return_val_if_fail (VALA_IS_CCODE_INCLUDE_DIRECTIVE (self), FALSE);
 	return self->priv->_local;
 }
 
 
-void vala_ccode_include_directive_set_local (ValaCCodeIncludeDirective* self, gboolean value)
-{
+void vala_ccode_include_directive_set_local (ValaCCodeIncludeDirective* self, gboolean value) {
 	g_return_if_fail (VALA_IS_CCODE_INCLUDE_DIRECTIVE (self));
 	self->priv->_local = value;
 }
 
 
-static void vala_ccode_include_directive_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec)
-{
+static void vala_ccode_include_directive_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
 	ValaCCodeIncludeDirective * self;
 	self = VALA_CCODE_INCLUDE_DIRECTIVE (object);
 	switch (property_id) {
@@ -128,8 +122,7 @@ static void vala_ccode_include_directive_get_property (GObject * object, guint p
 }
 
 
-static void vala_ccode_include_directive_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec)
-{
+static void vala_ccode_include_directive_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
 	ValaCCodeIncludeDirective * self;
 	self = VALA_CCODE_INCLUDE_DIRECTIVE (object);
 	switch (property_id) {
@@ -146,40 +139,32 @@ static void vala_ccode_include_directive_set_property (GObject * object, guint p
 }
 
 
-static void vala_ccode_include_directive_class_init (ValaCCodeIncludeDirectiveClass * klass)
-{
+static void vala_ccode_include_directive_class_init (ValaCCodeIncludeDirectiveClass * klass) {
 	vala_ccode_include_directive_parent_class = g_type_class_peek_parent (klass);
 	g_type_class_add_private (klass, sizeof (ValaCCodeIncludeDirectivePrivate));
 	G_OBJECT_CLASS (klass)->get_property = vala_ccode_include_directive_get_property;
 	G_OBJECT_CLASS (klass)->set_property = vala_ccode_include_directive_set_property;
 	G_OBJECT_CLASS (klass)->dispose = vala_ccode_include_directive_dispose;
 	VALA_CCODE_NODE_CLASS (klass)->write = vala_ccode_include_directive_real_write;
-	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_CCODE_INCLUDE_DIRECTIVE_FILENAME, g_param_spec_string ("filename", "foo", "bar", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT));
-	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_CCODE_INCLUDE_DIRECTIVE_LOCAL, g_param_spec_boolean ("local", "foo", "bar", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
+	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_CCODE_INCLUDE_DIRECTIVE_FILENAME, g_param_spec_string ("filename", "filename", "filename", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT));
+	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_CCODE_INCLUDE_DIRECTIVE_LOCAL, g_param_spec_boolean ("local", "local", "local", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
 }
 
 
-static void vala_ccode_include_directive_init (ValaCCodeIncludeDirective * self)
-{
+static void vala_ccode_include_directive_init (ValaCCodeIncludeDirective * self) {
 	self->priv = VALA_CCODE_INCLUDE_DIRECTIVE_GET_PRIVATE (self);
 }
 
 
-static void vala_ccode_include_directive_dispose (GObject * obj)
-{
+static void vala_ccode_include_directive_dispose (GObject * obj) {
 	ValaCCodeIncludeDirective * self;
-	ValaCCodeIncludeDirectiveClass * klass;
-	GObjectClass * parent_class;
 	self = VALA_CCODE_INCLUDE_DIRECTIVE (obj);
 	(self->priv->_filename = (g_free (self->priv->_filename), NULL));
-	klass = VALA_CCODE_INCLUDE_DIRECTIVE_CLASS (g_type_class_peek (VALA_TYPE_CCODE_INCLUDE_DIRECTIVE));
-	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
-	parent_class->dispose (obj);
+	G_OBJECT_CLASS (vala_ccode_include_directive_parent_class)->dispose (obj);
 }
 
 
-GType vala_ccode_include_directive_get_type (void)
-{
+GType vala_ccode_include_directive_get_type (void) {
 	static GType vala_ccode_include_directive_type_id = 0;
 	if (G_UNLIKELY (vala_ccode_include_directive_type_id == 0)) {
 		static const GTypeInfo g_define_type_info = { sizeof (ValaCCodeIncludeDirectiveClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_ccode_include_directive_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaCCodeIncludeDirective), 0, (GInstanceInitFunc) vala_ccode_include_directive_init };
