@@ -44,8 +44,7 @@ ValaCCodeMemberAccess* vala_ccode_member_access_new (ValaCCodeExpression* contai
 	GParameter * __params;
 	GParameter * __params_it;
 	ValaCCodeMemberAccess * self;
-	g_return_val_if_fail (VALA_IS_CCODE_EXPRESSION (container), NULL);
-	g_return_val_if_fail (member != NULL, NULL);
+	g_return_val_if_fail (container == NULL || VALA_IS_CCODE_EXPRESSION (container), NULL);
 	__params = g_new0 (GParameter, 2);
 	__params_it = __params;
 	(__params_it->name = "inner", g_value_init (&__params_it->value, VALA_TYPE_CCODE_EXPRESSION), g_value_set_object (&__params_it->value, container), __params_it++);
@@ -65,8 +64,7 @@ ValaCCodeMemberAccess* vala_ccode_member_access_new_pointer (ValaCCodeExpression
 	GParameter * __params;
 	GParameter * __params_it;
 	ValaCCodeMemberAccess * self;
-	g_return_val_if_fail (VALA_IS_CCODE_EXPRESSION (container), NULL);
-	g_return_val_if_fail (member != NULL, NULL);
+	g_return_val_if_fail (container == NULL || VALA_IS_CCODE_EXPRESSION (container), NULL);
 	__params = g_new0 (GParameter, 2);
 	__params_it = __params;
 	(__params_it->name = "inner", g_value_init (&__params_it->value, VALA_TYPE_CCODE_EXPRESSION), g_value_set_object (&__params_it->value, container), __params_it++);
@@ -85,7 +83,7 @@ ValaCCodeMemberAccess* vala_ccode_member_access_new_pointer (ValaCCodeExpression
 static void vala_ccode_member_access_real_write (ValaCCodeNode* base, ValaCCodeWriter* writer) {
 	ValaCCodeMemberAccess * self;
 	self = VALA_CCODE_MEMBER_ACCESS (base);
-	g_return_if_fail (VALA_IS_CCODE_WRITER (writer));
+	g_return_if_fail (writer == NULL || VALA_IS_CCODE_WRITER (writer));
 	vala_ccode_node_write (VALA_CCODE_NODE (vala_ccode_member_access_get_inner (self)), writer);
 	if (vala_ccode_member_access_get_is_pointer (self)) {
 		vala_ccode_writer_write_string (writer, "->");
@@ -103,10 +101,12 @@ ValaCCodeExpression* vala_ccode_member_access_get_inner (ValaCCodeMemberAccess*
 
 
 void vala_ccode_member_access_set_inner (ValaCCodeMemberAccess* self, ValaCCodeExpression* value) {
+	ValaCCodeExpression* _tmp2;
 	ValaCCodeExpression* _tmp1;
 	g_return_if_fail (VALA_IS_CCODE_MEMBER_ACCESS (self));
+	_tmp2 = NULL;
 	_tmp1 = NULL;
-	self->priv->_inner = (_tmp1 = g_object_ref (value), (self->priv->_inner == NULL ? NULL : (self->priv->_inner = (g_object_unref (self->priv->_inner), NULL))), _tmp1);
+	self->priv->_inner = (_tmp2 = (_tmp1 = value, (_tmp1 == NULL ? NULL : g_object_ref (_tmp1))), (self->priv->_inner == NULL ? NULL : (self->priv->_inner = (g_object_unref (self->priv->_inner), NULL))), _tmp2);
 }
 
 
@@ -117,10 +117,12 @@ char* vala_ccode_member_access_get_member_name (ValaCCodeMemberAccess* self) {
 
 
 void vala_ccode_member_access_set_member_name (ValaCCodeMemberAccess* self, const char* value) {
-	char* _tmp1;
+	char* _tmp2;
+	const char* _tmp1;
 	g_return_if_fail (VALA_IS_CCODE_MEMBER_ACCESS (self));
+	_tmp2 = NULL;
 	_tmp1 = NULL;
-	self->priv->_member_name = (_tmp1 = g_strdup (value), (self->priv->_member_name = (g_free (self->priv->_member_name), NULL)), _tmp1);
+	self->priv->_member_name = (_tmp2 = (_tmp1 = value, (_tmp1 == NULL ? NULL : g_strdup (_tmp1))), (self->priv->_member_name = (g_free (self->priv->_member_name), NULL)), _tmp2);
 }
 
 
