@@ -1,6 +1,6 @@
 /* valaccodemodifiers.vala
  *
- * Copyright (C) 2006-2007  Jürg Billeter, Raffaele Sandrini
+ * Copyright (C) 2006-2008  Jürg Billeter, Raffaele Sandrini
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -31,10 +31,11 @@ G_BEGIN_DECLS
 
 
 typedef enum  {
-	VALA_CCODE_MODIFIERS_NONE,
-	VALA_CCODE_MODIFIERS_STATIC,
-	VALA_CCODE_MODIFIERS_REGISTER,
-	VALA_CCODE_MODIFIERS_EXTERN
+	VALA_CCODE_MODIFIERS_NONE = 0,
+	VALA_CCODE_MODIFIERS_STATIC = 1 << 0,
+	VALA_CCODE_MODIFIERS_REGISTER = 1 << 1,
+	VALA_CCODE_MODIFIERS_EXTERN = 1 << 2,
+	VALA_CCODE_MODIFIERS_INLINE = 1 << 3
 } ValaCCodeModifiers;
 
 
