@@ -69,12 +69,12 @@ static void gee_array_list_set_capacity (GeeArrayList* self, gint value);
 #define GEE_ARRAY_LIST_ITERATOR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_ARRAY_LIST_TYPE_ITERATOR, GeeArrayListIteratorClass))
 
 struct _GeeArrayListIterator {
-	GObject parent;
+	GObject parent_instance;
 	GeeArrayListIteratorPrivate * priv;
 	gint _stamp;
 };
 struct _GeeArrayListIteratorClass {
-	GObjectClass parent;
+	GObjectClass parent_class;
 };
 struct _GeeArrayListIteratorPrivate {
 	GeeArrayList* _list;
@@ -284,7 +284,7 @@ void gee_array_list_set_equal_func (GeeArrayList* self, GEqualFunc value) {
 
 static GeeArrayListIterator* gee_array_list_iterator_new (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, GeeArrayList* list) {
 	GeeArrayListIterator * self;
-	g_return_val_if_fail (GEE_IS_ARRAY_LIST (list), NULL);
+	g_return_val_if_fail (list == NULL || GEE_IS_ARRAY_LIST (list), NULL);
 	self = g_object_newv (GEE_ARRAY_LIST_TYPE_ITERATOR, 0, NULL);
 	gee_array_list_iterator_set_list (self, list);
 	self->priv->g_type = g_type;
