@@ -2,7 +2,7 @@
  *
  * Copyright (C) 1995-1997  Peter Mattis, Spencer Kimball and Josh MacDonald
  * Copyright (C) 1997-2000  GLib Team and others
- * Copyright (C) 2007  Jürg Billeter
+ * Copyright (C) 2007-2008  Jürg Billeter
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -96,11 +96,11 @@ static void gee_hash_map_node_free (GeeHashMapNode* self);
 #define GEE_HASH_MAP_KEY_SET_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_HASH_MAP_TYPE_KEY_SET, GeeHashMapKeySetClass))
 
 struct _GeeHashMapKeySet {
-	GObject parent;
+	GObject parent_instance;
 	GeeHashMapKeySetPrivate * priv;
 };
 struct _GeeHashMapKeySetClass {
-	GObjectClass parent;
+	GObjectClass parent_class;
 };
 struct _GeeHashMapKeySetPrivate {
 	GeeHashMap* _map;
@@ -145,11 +145,11 @@ static GType gee_hash_map_key_set_get_type (void);
 #define GEE_HASH_MAP_KEY_ITERATOR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_HASH_MAP_TYPE_KEY_ITERATOR, GeeHashMapKeyIteratorClass))
 
 struct _GeeHashMapKeyIterator {
-	GObject parent;
+	GObject parent_instance;
 	GeeHashMapKeyIteratorPrivate * priv;
 };
 struct _GeeHashMapKeyIteratorClass {
-	GObjectClass parent;
+	GObjectClass parent_class;
 };
 struct _GeeHashMapKeyIteratorPrivate {
 	GeeHashMap* _map;
@@ -191,11 +191,11 @@ static GType gee_hash_map_key_iterator_get_type (void);
 #define GEE_HASH_MAP_VALUE_COLLECTION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_HASH_MAP_TYPE_VALUE_COLLECTION, GeeHashMapValueCollectionClass))
 
 struct _GeeHashMapValueCollection {
-	GObject parent;
+	GObject parent_instance;
 	GeeHashMapValueCollectionPrivate * priv;
 };
 struct _GeeHashMapValueCollectionClass {
-	GObjectClass parent;
+	GObjectClass parent_class;
 };
 struct _GeeHashMapValueCollectionPrivate {
 	GeeHashMap* _map;
@@ -239,11 +239,11 @@ static GType gee_hash_map_value_collection_get_type (void);
 #define GEE_HASH_MAP_VALUE_ITERATOR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_HASH_MAP_TYPE_VALUE_ITERATOR, GeeHashMapValueIteratorClass))
 
 struct _GeeHashMapValueIterator {
-	GObject parent;
+	GObject parent_instance;
 	GeeHashMapValueIteratorPrivate * priv;
 };
 struct _GeeHashMapValueIteratorClass {
-	GObjectClass parent;
+	GObjectClass parent_class;
 };
 struct _GeeHashMapValueIteratorPrivate {
 	GeeHashMap* _map;
@@ -318,8 +318,8 @@ static GeeHashMapNode** gee_hash_map_lookup_node (GeeHashMap* self, gconstpointe
 	g_return_val_if_fail (GEE_IS_HASH_MAP (self), NULL);
 	hash_value = self->priv->_key_hash_func (key);
 	node = &self->priv->_nodes[hash_value % self->priv->_array_size];
-	while ((*node) != NULL && (hash_value != (*node)->key_hash || !self->priv->_key_equal_func ((*node)->key, key))) {
-		node = &((*node)->next);
+	while ((*node) != NULL && (hash_value != (((GeeHashMapNode*) (*node)))->key_hash || !self->priv->_key_equal_func ((((GeeHashMapNode*) (*node)))->key, key))) {
+		node = &((((GeeHashMapNode*) (*node)))->next);
 	}
 	return node;
 }
@@ -338,7 +338,7 @@ static gpointer gee_hash_map_real_get (GeeMap* base, gconstpointer key) {
 	GeeHashMap * self;
 	GeeHashMapNode* node;
 	self = GEE_HASH_MAP (base);
-	node = *gee_hash_map_lookup_node (self, key);
+	node = ((GeeHashMapNode*) (*gee_hash_map_lookup_node (self, key)));
 	if (node != NULL) {
 		gconstpointer _tmp0;
 		return (_tmp0 = node->value, (_tmp0 == NULL || self->priv->v_dup_func == NULL ? ((gpointer) _tmp0) : self->priv->v_dup_func (((gpointer) _tmp0))));
@@ -356,7 +356,7 @@ static void gee_hash_map_real_set (GeeMap* base, gconstpointer key, gconstpointe
 	if (*node != NULL) {
 		gpointer _tmp1;
 		gconstpointer _tmp0;
-		(*node)->value = (_tmp1 = (_tmp0 = value, (_tmp0 == NULL || self->priv->v_dup_func == NULL ? ((gpointer) _tmp0) : self->priv->v_dup_func (((gpointer) _tmp0)))), ((*node)->value == NULL || self->priv->v_destroy_func == NULL ? NULL : ((*node)->value = (self->priv->v_destroy_func ((*node)->value), NULL))), _tmp1);
+		(((GeeHashMapNode*) (*node)))->value = (_tmp1 = (_tmp0 = value, (_tmp0 == NULL || self->priv->v_dup_func == NULL ? ((gpointer) _tmp0) : self->priv->v_dup_func (((gpointer) _tmp0)))), ((((GeeHashMapNode*) (*node)))->value == NULL || self->priv->v_destroy_func == NULL ? NULL : ((((GeeHashMapNode*) (*node)))->value = (self->priv->v_destroy_func ((((GeeHashMapNode*) (*node)))->value), NULL))), _tmp1);
 	} else {
 		guint hash_value;
 		gconstpointer _tmp3;
@@ -378,9 +378,9 @@ static gboolean gee_hash_map_real_remove (GeeMap* base, gconstpointer key) {
 	if (*node != NULL) {
 		gpointer _tmp0;
 		gpointer _tmp1;
-		(*node)->key = (_tmp0 = NULL, ((*node)->key == NULL || self->priv->k_destroy_func == NULL ? NULL : ((*node)->key = (self->priv->k_destroy_func ((*node)->key), NULL))), _tmp0);
-		(*node)->value = (_tmp1 = NULL, ((*node)->value == NULL || self->priv->v_destroy_func == NULL ? NULL : ((*node)->value = (self->priv->v_destroy_func ((*node)->value), NULL))), _tmp1);
-		*node = (*node)->next;
+		(((GeeHashMapNode*) (*node)))->key = (_tmp0 = NULL, ((((GeeHashMapNode*) (*node)))->key == NULL || self->priv->k_destroy_func == NULL ? NULL : ((((GeeHashMapNode*) (*node)))->key = (self->priv->k_destroy_func ((((GeeHashMapNode*) (*node)))->key), NULL))), _tmp0);
+		(((GeeHashMapNode*) (*node)))->value = (_tmp1 = NULL, ((((GeeHashMapNode*) (*node)))->value == NULL || self->priv->v_destroy_func == NULL ? NULL : ((((GeeHashMapNode*) (*node)))->value = (self->priv->v_destroy_func ((((GeeHashMapNode*) (*node)))->value), NULL))), _tmp1);
+		*node = (((GeeHashMapNode*) (*node)))->next;
 		self->priv->_nnodes--;
 		gee_hash_map_resize (self);
 		self->priv->_stamp++;
@@ -553,7 +553,7 @@ static void gee_hash_map_node_free (GeeHashMapNode* self) {
 
 static GeeHashMapKeySet* gee_hash_map_key_set_new (GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func, GeeHashMap* map) {
 	GeeHashMapKeySet * self;
-	g_return_val_if_fail (GEE_IS_HASH_MAP (map), NULL);
+	g_return_val_if_fail (map == NULL || GEE_IS_HASH_MAP (map), NULL);
 	self = g_object_newv (GEE_HASH_MAP_TYPE_KEY_SET, 0, NULL);
 	gee_hash_map_key_set_set_map (self, map);
 	self->priv->k_type = k_type;
@@ -733,7 +733,7 @@ static GType gee_hash_map_key_set_get_type (void) {
 
 static GeeHashMapKeyIterator* gee_hash_map_key_iterator_new (GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func, GeeHashMap* map) {
 	GeeHashMapKeyIterator * self;
-	g_return_val_if_fail (GEE_IS_HASH_MAP (map), NULL);
+	g_return_val_if_fail (map == NULL || GEE_IS_HASH_MAP (map), NULL);
 	self = g_object_newv (GEE_HASH_MAP_TYPE_KEY_ITERATOR, 0, NULL);
 	gee_hash_map_key_iterator_set_map (self, map);
 	self->priv->k_type = k_type;
@@ -875,7 +875,7 @@ static GType gee_hash_map_key_iterator_get_type (void) {
 
 static GeeHashMapValueCollection* gee_hash_map_value_collection_new (GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func, GeeHashMap* map) {
 	GeeHashMapValueCollection * self;
-	g_return_val_if_fail (GEE_IS_HASH_MAP (map), NULL);
+	g_return_val_if_fail (map == NULL || GEE_IS_HASH_MAP (map), NULL);
 	self = g_object_newv (GEE_HASH_MAP_TYPE_VALUE_COLLECTION, 0, NULL);
 	gee_hash_map_value_collection_set_map (self, map);
 	self->priv->k_type = k_type;
@@ -899,6 +899,7 @@ static gboolean gee_hash_map_value_collection_real_add (GeeCollection* base, gco
 	GeeHashMapValueCollection * self;
 	self = GEE_HASH_MAP_VALUE_COLLECTION (base);
 	g_assert_not_reached ();
+	return FALSE;
 }
 
 
@@ -913,6 +914,7 @@ static gboolean gee_hash_map_value_collection_real_remove (GeeCollection* base,
 	GeeHashMapValueCollection * self;
 	self = GEE_HASH_MAP_VALUE_COLLECTION (base);
 	g_assert_not_reached ();
+	return FALSE;
 }
 
 
@@ -931,7 +933,6 @@ static gboolean gee_hash_map_value_collection_real_contains (GeeCollection* base
 		}
 	}
 	return (_tmp3 = FALSE, (it == NULL ? NULL : (it = (g_object_unref (it), NULL))), _tmp3);
-	(it == NULL ? NULL : (it = (g_object_unref (it), NULL)));
 }
 
 
@@ -1058,7 +1059,7 @@ static GType gee_hash_map_value_collection_get_type (void) {
 
 static GeeHashMapValueIterator* gee_hash_map_value_iterator_new (GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func, GeeHashMap* map) {
 	GeeHashMapValueIterator * self;
-	g_return_val_if_fail (GEE_IS_HASH_MAP (map), NULL);
+	g_return_val_if_fail (map == NULL || GEE_IS_HASH_MAP (map), NULL);
 	self = g_object_newv (GEE_HASH_MAP_TYPE_VALUE_ITERATOR, 0, NULL);
 	gee_hash_map_value_iterator_set_map (self, map);
 	self->priv->k_type = k_type;
