@@ -5,7 +5,7 @@
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * version 2.1 of the License, or (at your option) any later version.
 
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -20,13 +20,12 @@
  * 	Jürg Billeter <j@bitron.ch>
  */
 
-#include "valaccodecompiler.h"
+#include <gobject/valaccodecompiler.h>
 #include <vala/valareport.h>
 #include <vala/valasourcereference.h>
 #include <gee/collection.h>
 #include <vala/valasourcefile.h>
 #include <glib/gstdio.h>
-#include <gobject/valaccodecompiler.h>
 
 enum  {
 	VALA_CCODE_COMPILER_DUMMY_PROPERTY
@@ -35,16 +34,14 @@ static gboolean vala_ccode_compiler_package_exists (const char* package_name);
 static gpointer vala_ccode_compiler_parent_class = NULL;
 
 
-ValaCCodeCompiler* vala_ccode_compiler_new (void)
-{
+ValaCCodeCompiler* vala_ccode_compiler_new (void) {
 	ValaCCodeCompiler * self;
 	self = g_object_newv (VALA_TYPE_CCODE_COMPILER, 0, NULL);
 	return self;
 }
 
 
-static gboolean vala_ccode_compiler_package_exists (const char* package_name)
-{
+static gboolean vala_ccode_compiler_package_exists (const char* package_name) {
 	GError * inner_error;
 	char* pc;
 	gint exit_status;
@@ -87,23 +84,22 @@ static gboolean vala_ccode_compiler_package_exists (const char* package_name)
  *
  * @param context a code context
  */
-void vala_ccode_compiler_compile (ValaCCodeCompiler* self, ValaCodeContext* context, const char* cc_command, char** cc_options)
-{
+void vala_ccode_compiler_compile (ValaCCodeCompiler* self, ValaCodeContext* context, const char* cc_command, char** cc_options) {
 	GError * inner_error;
 	char* pc;
 	char* _tmp1;
 	char* pkgflags;
 	gint exit_status;
-	const char* _tmp6;
+	const char* _tmp8;
 	char* cmdline;
-	char* _tmp9;
-	char* _tmp8;
-	char* _tmp17;
-	char* _tmp16;
+	char* _tmp11;
+	char* _tmp10;
+	char* _tmp19;
+	char* _tmp18;
 	GeeCollection* source_files;
 	GeeCollection* c_source_files;
 	g_return_if_fail (VALA_IS_CCODE_COMPILER (self));
-	g_return_if_fail (VALA_IS_CODE_CONTEXT (context));
+	g_return_if_fail (context == NULL || VALA_IS_CODE_CONTEXT (context));
 	inner_error = NULL;
 	pc = g_strdup ("pkg-config --cflags");
 	if (!vala_code_context_get_compile_only (context)) {
@@ -144,7 +140,10 @@ void vala_ccode_compiler_compile (ValaCCodeCompiler* self, ValaCodeContext* cont
 	pkgflags = NULL;
 	exit_status = 0;
 	{
-		g_spawn_command_line_sync (pc, &pkgflags, NULL, &exit_status, &inner_error);
+		gboolean _tmp6;
+		char* _tmp5;
+		_tmp5 = NULL;
+		(_tmp6 = g_spawn_command_line_sync (pc, &_tmp5, NULL, &exit_status, &inner_error), (pkgflags = (g_free (pkgflags), NULL)), pkgflags = _tmp5, _tmp6);
 		if (inner_error != NULL) {
 			if (inner_error->domain == G_SPAWN_ERROR) {
 				goto __catch1_g_spawn_error;
@@ -153,10 +152,10 @@ void vala_ccode_compiler_compile (ValaCCodeCompiler* self, ValaCodeContext* cont
 			g_clear_error (&inner_error);
 		}
 		if (exit_status != 0) {
-			char* _tmp5;
-			_tmp5 = NULL;
-			vala_report_error (NULL, (_tmp5 = g_strdup_printf ("pkg-config exited with status %d", exit_status)));
-			(_tmp5 = (g_free (_tmp5), NULL));
+			char* _tmp7;
+			_tmp7 = NULL;
+			vala_report_error (NULL, (_tmp7 = g_strdup_printf ("pkg-config exited with status %d", exit_status)));
+			(_tmp7 = (g_free (_tmp7), NULL));
 			(pc = (g_free (pc), NULL));
 			(pkgflags = (g_free (pkgflags), NULL));
 			return;
@@ -181,48 +180,48 @@ void vala_ccode_compiler_compile (ValaCCodeCompiler* self, ValaCodeContext* cont
 	if (cc_command == NULL) {
 		cc_command = "cc";
 	}
-	_tmp6 = NULL;
-	cmdline = (_tmp6 = cc_command, (_tmp6 == NULL ? NULL : g_strdup (_tmp6)));
-	if (vala_code_context_get_debug (context)) {
-		char* _tmp7;
-		_tmp7 = NULL;
-		cmdline = (_tmp7 = g_strconcat (cmdline, (" -g"), NULL), (cmdline = (g_free (cmdline), NULL)), _tmp7);
-	}
-	_tmp9 = NULL;
 	_tmp8 = NULL;
-	cmdline = (_tmp9 = g_strconcat (cmdline, (_tmp8 = (g_strdup_printf (" -O%d", vala_code_context_get_optlevel (context)))), NULL), (cmdline = (g_free (cmdline), NULL)), _tmp9);
-	(_tmp8 = (g_free (_tmp8), NULL));
+	cmdline = (_tmp8 = cc_command, (_tmp8 == NULL ? NULL : g_strdup (_tmp8)));
+	if (vala_code_context_get_debug (context)) {
+		char* _tmp9;
+		_tmp9 = NULL;
+		cmdline = (_tmp9 = g_strconcat (cmdline, (" -g"), NULL), (cmdline = (g_free (cmdline), NULL)), _tmp9);
+	}
+	_tmp11 = NULL;
+	_tmp10 = NULL;
+	cmdline = (_tmp11 = g_strconcat (cmdline, (_tmp10 = (g_strdup_printf (" -O%d", vala_code_context_get_optlevel (context)))), NULL), (cmdline = (g_free (cmdline), NULL)), _tmp11);
+	(_tmp10 = (g_free (_tmp10), NULL));
 	if (vala_code_context_get_compile_only (context)) {
-		char* _tmp10;
-		_tmp10 = NULL;
-		cmdline = (_tmp10 = g_strconcat (cmdline, (" -c"), NULL), (cmdline = (g_free (cmdline), NULL)), _tmp10);
+		char* _tmp12;
+		_tmp12 = NULL;
+		cmdline = (_tmp12 = g_strconcat (cmdline, (" -c"), NULL), (cmdline = (g_free (cmdline), NULL)), _tmp12);
 	} else {
 		if (vala_code_context_get_output (context) != NULL) {
-			const char* _tmp11;
+			const char* _tmp13;
 			char* output;
+			char* _tmp17;
+			char* _tmp16;
 			char* _tmp15;
-			char* _tmp14;
-			char* _tmp13;
-			_tmp11 = NULL;
-			output = (_tmp11 = vala_code_context_get_output (context), (_tmp11 == NULL ? NULL : g_strdup (_tmp11)));
-			if (vala_code_context_get_directory (context) != NULL && g_utf8_collate (vala_code_context_get_directory (context), "") != 0) {
-				char* _tmp12;
-				_tmp12 = NULL;
-				output = (_tmp12 = g_strdup_printf ("%s/%s", vala_code_context_get_directory (context), vala_code_context_get_output (context)), (output = (g_free (output), NULL)), _tmp12);
-			}
-			_tmp15 = NULL;
-			_tmp14 = NULL;
 			_tmp13 = NULL;
-			cmdline = (_tmp15 = g_strconcat (cmdline, (_tmp14 = (g_strconcat (" -o ", (_tmp13 = g_shell_quote (output)), NULL))), NULL), (cmdline = (g_free (cmdline), NULL)), _tmp15);
-			(_tmp14 = (g_free (_tmp14), NULL));
-			(_tmp13 = (g_free (_tmp13), NULL));
+			output = (_tmp13 = vala_code_context_get_output (context), (_tmp13 == NULL ? NULL : g_strdup (_tmp13)));
+			if (vala_code_context_get_directory (context) != NULL && g_utf8_collate (vala_code_context_get_directory (context), "") != 0) {
+				char* _tmp14;
+				_tmp14 = NULL;
+				output = (_tmp14 = g_strdup_printf ("%s/%s", vala_code_context_get_directory (context), vala_code_context_get_output (context)), (output = (g_free (output), NULL)), _tmp14);
+			}
+			_tmp17 = NULL;
+			_tmp16 = NULL;
+			_tmp15 = NULL;
+			cmdline = (_tmp17 = g_strconcat (cmdline, (_tmp16 = (g_strconcat (" -o ", (_tmp15 = g_shell_quote (output)), NULL))), NULL), (cmdline = (g_free (cmdline), NULL)), _tmp17);
+			(_tmp16 = (g_free (_tmp16), NULL));
+			(_tmp15 = (g_free (_tmp15), NULL));
 			(output = (g_free (output), NULL));
 		}
 	}
-	_tmp17 = NULL;
-	_tmp16 = NULL;
-	cmdline = (_tmp17 = g_strconcat (cmdline, (_tmp16 = (g_strconcat (" ", pkgflags, NULL))), NULL), (cmdline = (g_free (cmdline), NULL)), _tmp17);
-	(_tmp16 = (g_free (_tmp16), NULL));
+	_tmp19 = NULL;
+	_tmp18 = NULL;
+	cmdline = (_tmp19 = g_strconcat (cmdline, (_tmp18 = (g_strconcat (" ", pkgflags, NULL))), NULL), (cmdline = (g_free (cmdline), NULL)), _tmp19);
+	(_tmp18 = (g_free (_tmp18), NULL));
 	if (cc_options != NULL) {
 		{
 			char** cc_option_collection;
@@ -231,17 +230,17 @@ void vala_ccode_compiler_compile (ValaCCodeCompiler* self, ValaCodeContext* cont
 			cc_option_collection = cc_options;
 			cc_option_collection_length1 = -1;
 			for (cc_option_it = cc_option_collection; *cc_option_it != NULL; cc_option_it = cc_option_it + 1) {
-				const char* _tmp20;
+				const char* _tmp22;
 				char* cc_option;
-				_tmp20 = NULL;
-				cc_option = (_tmp20 = *cc_option_it, (_tmp20 == NULL ? NULL : g_strdup (_tmp20)));
+				_tmp22 = NULL;
+				cc_option = (_tmp22 = *cc_option_it, (_tmp22 == NULL ? NULL : g_strdup (_tmp22)));
 				{
-					char* _tmp19;
-					char* _tmp18;
-					_tmp19 = NULL;
-					_tmp18 = NULL;
-					cmdline = (_tmp19 = g_strconcat (cmdline, (_tmp18 = (g_strconcat (" ", cc_option, NULL))), NULL), (cmdline = (g_free (cmdline), NULL)), _tmp19);
-					(_tmp18 = (g_free (_tmp18), NULL));
+					char* _tmp21;
+					char* _tmp20;
+					_tmp21 = NULL;
+					_tmp20 = NULL;
+					cmdline = (_tmp21 = g_strconcat (cmdline, (_tmp20 = (g_strconcat (" ", cc_option, NULL))), NULL), (cmdline = (g_free (cmdline), NULL)), _tmp21);
+					(_tmp20 = (g_free (_tmp20), NULL));
 					(cc_option = (g_free (cc_option), NULL));
 				}
 			}
@@ -249,18 +248,15 @@ void vala_ccode_compiler_compile (ValaCCodeCompiler* self, ValaCodeContext* cont
 	}
 	/* make sure include files can be found if -d is used */
 	if (vala_code_context_get_directory (context) != NULL && g_utf8_collate (vala_code_context_get_directory (context), "") != 0) {
+		char* _tmp25;
 		char* _tmp24;
 		char* _tmp23;
-		char* _tmp22;
-		char* _tmp21;
+		_tmp25 = NULL;
 		_tmp24 = NULL;
 		_tmp23 = NULL;
-		_tmp22 = NULL;
-		_tmp21 = NULL;
-		cmdline = (_tmp24 = g_strconcat (cmdline, (_tmp23 = (g_strconcat (" -I", (_tmp22 = g_shell_quote ((_tmp21 = g_strdup_printf ("%s/..", vala_code_context_get_directory (context))))), NULL))), NULL), (cmdline = (g_free (cmdline), NULL)), _tmp24);
+		cmdline = (_tmp25 = g_strconcat (cmdline, (_tmp24 = (g_strconcat (" -I", (_tmp23 = g_shell_quote (vala_code_context_get_directory (context))), NULL))), NULL), (cmdline = (g_free (cmdline), NULL)), _tmp25);
+		(_tmp24 = (g_free (_tmp24), NULL));
 		(_tmp23 = (g_free (_tmp23), NULL));
-		(_tmp22 = (g_free (_tmp22), NULL));
-		(_tmp21 = (g_free (_tmp21), NULL));
 	}
 	/* we're only interested in non-pkg source files */
 	source_files = vala_code_context_get_source_files (context);
@@ -274,18 +270,18 @@ void vala_ccode_compiler_compile (ValaCCodeCompiler* self, ValaCodeContext* cont
 			file = gee_iterator_get (file_it);
 			{
 				if (!vala_source_file_get_pkg (file)) {
+					char* _tmp29;
 					char* _tmp28;
 					char* _tmp27;
 					char* _tmp26;
-					char* _tmp25;
+					_tmp29 = NULL;
 					_tmp28 = NULL;
 					_tmp27 = NULL;
 					_tmp26 = NULL;
-					_tmp25 = NULL;
-					cmdline = (_tmp28 = g_strconcat (cmdline, (_tmp27 = (g_strconcat (" ", (_tmp26 = g_shell_quote ((_tmp25 = vala_source_file_get_csource_filename (file)))), NULL))), NULL), (cmdline = (g_free (cmdline), NULL)), _tmp28);
+					cmdline = (_tmp29 = g_strconcat (cmdline, (_tmp28 = (g_strconcat (" ", (_tmp27 = g_shell_quote ((_tmp26 = vala_source_file_get_csource_filename (file)))), NULL))), NULL), (cmdline = (g_free (cmdline), NULL)), _tmp29);
+					(_tmp28 = (g_free (_tmp28), NULL));
 					(_tmp27 = (g_free (_tmp27), NULL));
 					(_tmp26 = (g_free (_tmp26), NULL));
-					(_tmp25 = (g_free (_tmp25), NULL));
 				}
 				(file == NULL ? NULL : (file = (g_object_unref (file), NULL)));
 			}
@@ -302,15 +298,15 @@ void vala_ccode_compiler_compile (ValaCCodeCompiler* self, ValaCodeContext* cont
 			char* file;
 			file = gee_iterator_get (file_it);
 			{
+				char* _tmp32;
 				char* _tmp31;
 				char* _tmp30;
-				char* _tmp29;
+				_tmp32 = NULL;
 				_tmp31 = NULL;
 				_tmp30 = NULL;
-				_tmp29 = NULL;
-				cmdline = (_tmp31 = g_strconcat (cmdline, (_tmp30 = (g_strconcat (" ", (_tmp29 = g_shell_quote (file)), NULL))), NULL), (cmdline = (g_free (cmdline), NULL)), _tmp31);
+				cmdline = (_tmp32 = g_strconcat (cmdline, (_tmp31 = (g_strconcat (" ", (_tmp30 = g_shell_quote (file)), NULL))), NULL), (cmdline = (g_free (cmdline), NULL)), _tmp32);
+				(_tmp31 = (g_free (_tmp31), NULL));
 				(_tmp30 = (g_free (_tmp30), NULL));
-				(_tmp29 = (g_free (_tmp29), NULL));
 				(file = (g_free (file), NULL));
 			}
 		}
@@ -326,10 +322,10 @@ void vala_ccode_compiler_compile (ValaCCodeCompiler* self, ValaCodeContext* cont
 			g_clear_error (&inner_error);
 		}
 		if (exit_status != 0) {
-			char* _tmp32;
-			_tmp32 = NULL;
-			vala_report_error (NULL, (_tmp32 = g_strdup_printf ("cc exited with status %d", exit_status)));
-			(_tmp32 = (g_free (_tmp32), NULL));
+			char* _tmp33;
+			_tmp33 = NULL;
+			vala_report_error (NULL, (_tmp33 = g_strdup_printf ("cc exited with status %d", exit_status)));
+			(_tmp33 = (g_free (_tmp33), NULL));
 		}
 	}
 	goto __finally2;
@@ -356,17 +352,17 @@ void vala_ccode_compiler_compile (ValaCCodeCompiler* self, ValaCodeContext* cont
 				/* remove generated C source and header files */
 				if (!vala_source_file_get_pkg (file)) {
 					if (!vala_code_context_get_save_csources (context)) {
-						char* _tmp33;
-						_tmp33 = NULL;
-						g_unlink ((_tmp33 = vala_source_file_get_csource_filename (file)));
-						(_tmp33 = (g_free (_tmp33), NULL));
-					}
-					if (!vala_code_context_get_save_cheaders (context)) {
 						char* _tmp34;
 						_tmp34 = NULL;
-						g_unlink ((_tmp34 = vala_source_file_get_cheader_filename (file)));
+						g_unlink ((_tmp34 = vala_source_file_get_csource_filename (file)));
 						(_tmp34 = (g_free (_tmp34), NULL));
 					}
+					if (!vala_code_context_get_save_cheaders (context)) {
+						char* _tmp35;
+						_tmp35 = NULL;
+						g_unlink ((_tmp35 = vala_source_file_get_cheader_filename (file)));
+						(_tmp35 = (g_free (_tmp35), NULL));
+					}
 				}
 				(file == NULL ? NULL : (file = (g_object_unref (file), NULL)));
 			}
@@ -381,19 +377,16 @@ void vala_ccode_compiler_compile (ValaCCodeCompiler* self, ValaCodeContext* cont
 }
 
 
-static void vala_ccode_compiler_class_init (ValaCCodeCompilerClass * klass)
-{
+static void vala_ccode_compiler_class_init (ValaCCodeCompilerClass * klass) {
 	vala_ccode_compiler_parent_class = g_type_class_peek_parent (klass);
 }
 
 
-static void vala_ccode_compiler_init (ValaCCodeCompiler * self)
-{
+static void vala_ccode_compiler_init (ValaCCodeCompiler * self) {
 }
 
 
-GType vala_ccode_compiler_get_type (void)
-{
+GType vala_ccode_compiler_get_type (void) {
 	static GType vala_ccode_compiler_type_id = 0;
 	if (G_UNLIKELY (vala_ccode_compiler_type_id == 0)) {
 		static const GTypeInfo g_define_type_info = { sizeof (ValaCCodeCompilerClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_ccode_compiler_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaCCodeCompiler), 0, (GInstanceInitFunc) vala_ccode_compiler_init };
