@@ -47,11 +47,11 @@ typedef struct _ValaCCodeCompilerPrivate ValaCCodeCompilerPrivate;
  * Interface to the C compiler.
  */
 struct _ValaCCodeCompiler {
-	GObject parent;
+	GObject parent_instance;
 	ValaCCodeCompilerPrivate * priv;
 };
 struct _ValaCCodeCompilerClass {
-	GObjectClass parent;
+	GObjectClass parent_class;
 };
 
 ValaCCodeCompiler* vala_ccode_compiler_new (void);
