@@ -25,7 +25,7 @@
 
 #include <glib.h>
 #include <glib-object.h>
-#include <vala/valadatatype.h>
+#include <vala/valatypesymbol.h>
 #include <stdlib.h>
 #include <string.h>
 #include <ccode/valaccodefragment.h>
@@ -34,12 +34,12 @@
 G_BEGIN_DECLS
 
 
-#define VALA_TYPE_TYPE_REGISTER_FUNCTION (vala_type_register_function_get_type ())
-#define VALA_TYPE_REGISTER_FUNCTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_TYPE_REGISTER_FUNCTION, ValaTypeRegisterFunction))
-#define VALA_TYPE_REGISTER_FUNCTION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_TYPE_REGISTER_FUNCTION, ValaTypeRegisterFunctionClass))
-#define VALA_IS_TYPE_REGISTER_FUNCTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_TYPE_REGISTER_FUNCTION))
-#define VALA_IS_TYPE_REGISTER_FUNCTION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_TYPE_REGISTER_FUNCTION))
-#define VALA_TYPE_REGISTER_FUNCTION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_TYPE_REGISTER_FUNCTION, ValaTypeRegisterFunctionClass))
+#define VALA_TYPE_TYPEREGISTER_FUNCTION (vala_typeregister_function_get_type ())
+#define VALA_TYPEREGISTER_FUNCTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_TYPEREGISTER_FUNCTION, ValaTypeRegisterFunction))
+#define VALA_TYPEREGISTER_FUNCTION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_TYPEREGISTER_FUNCTION, ValaTypeRegisterFunctionClass))
+#define VALA_IS_TYPEREGISTER_FUNCTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_TYPEREGISTER_FUNCTION))
+#define VALA_IS_TYPEREGISTER_FUNCTION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_TYPEREGISTER_FUNCTION))
+#define VALA_TYPEREGISTER_FUNCTION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_TYPEREGISTER_FUNCTION, ValaTypeRegisterFunctionClass))
 
 typedef struct _ValaTypeRegisterFunction ValaTypeRegisterFunction;
 typedef struct _ValaTypeRegisterFunctionClass ValaTypeRegisterFunctionClass;
@@ -49,12 +49,12 @@ typedef struct _ValaTypeRegisterFunctionPrivate ValaTypeRegisterFunctionPrivate;
  * C function to register a type at runtime.
  */
 struct _ValaTypeRegisterFunction {
-	GObject parent;
+	GObject parent_instance;
 	ValaTypeRegisterFunctionPrivate * priv;
 };
 struct _ValaTypeRegisterFunctionClass {
-	GObjectClass parent;
-	ValaDataType* (*get_type_declaration) (ValaTypeRegisterFunction* self);
+	GObjectClass parent_class;
+	ValaTypesymbol* (*get_type_declaration) (ValaTypeRegisterFunction* self);
 	char* (*get_type_struct_name) (ValaTypeRegisterFunction* self);
 	char* (*get_base_init_func_name) (ValaTypeRegisterFunction* self);
 	char* (*get_class_init_func_name) (ValaTypeRegisterFunction* self);
@@ -67,21 +67,21 @@ struct _ValaTypeRegisterFunctionClass {
 	ValaSymbolAccessibility (*get_accessibility) (ValaTypeRegisterFunction* self);
 };
 
-void vala_type_register_function_init_from_type (ValaTypeRegisterFunction* self, gboolean plugin);
-ValaDataType* vala_type_register_function_get_type_declaration (ValaTypeRegisterFunction* self);
-char* vala_type_register_function_get_type_struct_name (ValaTypeRegisterFunction* self);
-char* vala_type_register_function_get_base_init_func_name (ValaTypeRegisterFunction* self);
-char* vala_type_register_function_get_class_init_func_name (ValaTypeRegisterFunction* self);
-char* vala_type_register_function_get_instance_struct_size (ValaTypeRegisterFunction* self);
-char* vala_type_register_function_get_instance_init_func_name (ValaTypeRegisterFunction* self);
-char* vala_type_register_function_get_parent_type_name (ValaTypeRegisterFunction* self);
-char* vala_type_register_function_get_type_flags (ValaTypeRegisterFunction* self);
-ValaCCodeFragment* vala_type_register_function_get_type_interface_init_declaration (ValaTypeRegisterFunction* self);
-ValaCCodeFragment* vala_type_register_function_get_type_interface_init_statements (ValaTypeRegisterFunction* self);
-ValaCCodeFragment* vala_type_register_function_get_declaration (ValaTypeRegisterFunction* self);
-ValaCCodeFragment* vala_type_register_function_get_definition (ValaTypeRegisterFunction* self);
-ValaSymbolAccessibility vala_type_register_function_get_accessibility (ValaTypeRegisterFunction* self);
-GType vala_type_register_function_get_type (void);
+void vala_typeregister_function_init_from_type (ValaTypeRegisterFunction* self, gboolean plugin);
+ValaTypesymbol* vala_typeregister_function_get_type_declaration (ValaTypeRegisterFunction* self);
+char* vala_typeregister_function_get_type_struct_name (ValaTypeRegisterFunction* self);
+char* vala_typeregister_function_get_base_init_func_name (ValaTypeRegisterFunction* self);
+char* vala_typeregister_function_get_class_init_func_name (ValaTypeRegisterFunction* self);
+char* vala_typeregister_function_get_instance_struct_size (ValaTypeRegisterFunction* self);
+char* vala_typeregister_function_get_instance_init_func_name (ValaTypeRegisterFunction* self);
+char* vala_typeregister_function_get_parent_type_name (ValaTypeRegisterFunction* self);
+char* vala_typeregister_function_get_type_flags (ValaTypeRegisterFunction* self);
+ValaCCodeFragment* vala_typeregister_function_get_type_interface_init_declaration (ValaTypeRegisterFunction* self);
+ValaCCodeFragment* vala_typeregister_function_get_type_interface_init_statements (ValaTypeRegisterFunction* self);
+ValaCCodeFragment* vala_typeregister_function_get_declaration (ValaTypeRegisterFunction* self);
+ValaCCodeFragment* vala_typeregister_function_get_definition (ValaTypeRegisterFunction* self);
+ValaSymbolAccessibility vala_typeregister_function_get_accessibility (ValaTypeRegisterFunction* self);
+GType vala_typeregister_function_get_type (void);
 
 G_END_DECLS
 
