@@ -7,7 +7,6 @@
 #include <vala/valablock.h>
 #include <vala/valabooleanliteral.h>
 #include <vala/valabreakstatement.h>
-#include <vala/valacallback.h>
 #include <vala/valacastexpression.h>
 #include <vala/valacatchclause.h>
 #include <vala/valacharacterliteral.h>
@@ -18,7 +17,9 @@
 #include <vala/valaconstructor.h>
 #include <vala/valacontinuestatement.h>
 #include <vala/valacreationmethod.h>
+#include <vala/valadatatype.h>
 #include <vala/valadeclarationstatement.h>
+#include <vala/valadelegate.h>
 #include <vala/valadestructor.h>
 #include <vala/valadostatement.h>
 #include <vala/valaelementaccess.h>
@@ -52,6 +53,7 @@
 #include <vala/valaparenthesizedexpression.h>
 #include <vala/valaparser.h>
 #include <vala/valapointerindirection.h>
+#include <vala/valapointertype.h>
 #include <vala/valapostfixexpression.h>
 #include <vala/valaproperty.h>
 #include <vala/valapropertyaccessor.h>
@@ -74,7 +76,9 @@
 #include <vala/valatypecheck.h>
 #include <vala/valatypeofexpression.h>
 #include <vala/valatypeparameter.h>
-#include <vala/valatypereference.h>
+#include <vala/valatypesymbol.h>
 #include <vala/valaunaryexpression.h>
+#include <vala/valaunresolvedtype.h>
 #include <vala/valavariabledeclarator.h>
+#include <vala/valavoidtype.h>
 #include <vala/valawhilestatement.h>
