@@ -52,7 +52,6 @@ static void vala_attribute_dispose (GObject * obj);
  */
 ValaAttribute* vala_attribute_new (const char* name, ValaSourceReference* source_reference) {
 	ValaAttribute * self;
-	g_return_val_if_fail (name != NULL, NULL);
 	g_return_val_if_fail (source_reference == NULL || VALA_IS_SOURCE_REFERENCE (source_reference), NULL);
 	self = g_object_newv (VALA_TYPE_ATTRIBUTE, 0, NULL);
 	vala_attribute_set_name (self, name);
@@ -68,7 +67,7 @@ ValaAttribute* vala_attribute_new (const char* name, ValaSourceReference* source
  */
 void vala_attribute_add_argument (ValaAttribute* self, ValaNamedArgument* arg) {
 	g_return_if_fail (VALA_IS_ATTRIBUTE (self));
-	g_return_if_fail (VALA_IS_NAMED_ARGUMENT (arg));
+	g_return_if_fail (arg == NULL || VALA_IS_NAMED_ARGUMENT (arg));
 	gee_collection_add (GEE_COLLECTION (self->args), arg);
 }
 
@@ -81,7 +80,6 @@ void vala_attribute_add_argument (ValaAttribute* self, ValaNamedArgument* arg) {
  */
 gboolean vala_attribute_has_argument (ValaAttribute* self, const char* name) {
 	g_return_val_if_fail (VALA_IS_ATTRIBUTE (self), FALSE);
-	g_return_val_if_fail (name != NULL, FALSE);
 	{
 		GeeList* arg_collection;
 		GeeIterator* arg_it;
@@ -113,7 +111,6 @@ gboolean vala_attribute_has_argument (ValaAttribute* self, const char* name) {
  */
 char* vala_attribute_get_string (ValaAttribute* self, const char* name) {
 	g_return_val_if_fail (VALA_IS_ATTRIBUTE (self), NULL);
-	g_return_val_if_fail (name != NULL, NULL);
 	{
 		GeeList* arg_collection;
 		GeeIterator* arg_it;
@@ -155,7 +152,6 @@ char* vala_attribute_get_string (ValaAttribute* self, const char* name) {
  */
 gint vala_attribute_get_integer (ValaAttribute* self, const char* name) {
 	g_return_val_if_fail (VALA_IS_ATTRIBUTE (self), 0);
-	g_return_val_if_fail (name != NULL, 0);
 	{
 		GeeList* arg_collection;
 		GeeIterator* arg_it;
@@ -196,7 +192,6 @@ gint vala_attribute_get_integer (ValaAttribute* self, const char* name) {
  */
 gboolean vala_attribute_get_bool (ValaAttribute* self, const char* name) {
 	g_return_val_if_fail (VALA_IS_ATTRIBUTE (self), FALSE);
-	g_return_val_if_fail (name != NULL, FALSE);
 	{
 		GeeList* arg_collection;
 		GeeIterator* arg_it;
@@ -236,10 +231,12 @@ char* vala_attribute_get_name (ValaAttribute* self) {
 
 
 void vala_attribute_set_name (ValaAttribute* self, const char* value) {
-	char* _tmp1;
+	char* _tmp2;
+	const char* _tmp1;
 	g_return_if_fail (VALA_IS_ATTRIBUTE (self));
+	_tmp2 = NULL;
 	_tmp1 = NULL;
-	self->priv->_name = (_tmp1 = g_strdup (value), (self->priv->_name = (g_free (self->priv->_name), NULL)), _tmp1);
+	self->priv->_name = (_tmp2 = (_tmp1 = value, (_tmp1 == NULL ? NULL : g_strdup (_tmp1))), (self->priv->_name = (g_free (self->priv->_name), NULL)), _tmp2);
 }
 
 
