@@ -29,6 +29,7 @@ enum  {
 };
 static void vala_base_access_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor);
 static char* vala_base_access_real_to_string (ValaCodeNode* base);
+static gboolean vala_base_access_real_is_pure (ValaExpression* base);
 static gpointer vala_base_access_parent_class = NULL;
 
 
@@ -50,7 +51,7 @@ ValaBaseAccess* vala_base_access_new (ValaSourceReference* source) {
 static void vala_base_access_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor) {
 	ValaBaseAccess * self;
 	self = VALA_BASE_ACCESS (base);
-	g_return_if_fail (VALA_IS_CODE_VISITOR (visitor));
+	g_return_if_fail (visitor == NULL || VALA_IS_CODE_VISITOR (visitor));
 	vala_code_visitor_visit_base_access (visitor, self);
 }
 
@@ -62,10 +63,18 @@ static char* vala_base_access_real_to_string (ValaCodeNode* base) {
 }
 
 
+static gboolean vala_base_access_real_is_pure (ValaExpression* base) {
+	ValaBaseAccess * self;
+	self = VALA_BASE_ACCESS (base);
+	return TRUE;
+}
+
+
 static void vala_base_access_class_init (ValaBaseAccessClass * klass) {
 	vala_base_access_parent_class = g_type_class_peek_parent (klass);
 	VALA_CODE_NODE_CLASS (klass)->accept = vala_base_access_real_accept;
 	VALA_CODE_NODE_CLASS (klass)->to_string = vala_base_access_real_to_string;
+	VALA_EXPRESSION_CLASS (klass)->is_pure = vala_base_access_real_is_pure;
 }
 
 
