@@ -66,7 +66,7 @@ ValaBlock* vala_block_new (ValaSourceReference* source_reference) {
  */
 void vala_block_add_statement (ValaBlock* self, ValaStatement* stmt) {
 	g_return_if_fail (VALA_IS_BLOCK (self));
-	g_return_if_fail (VALA_IS_STATEMENT (stmt));
+	g_return_if_fail (stmt == NULL || VALA_IS_STATEMENT (stmt));
 	gee_collection_add (GEE_COLLECTION (self->priv->statement_list), stmt);
 }
 
@@ -89,7 +89,7 @@ GeeCollection* vala_block_get_statements (ValaBlock* self) {
  */
 void vala_block_add_local_variable (ValaBlock* self, ValaVariableDeclarator* decl) {
 	g_return_if_fail (VALA_IS_BLOCK (self));
-	g_return_if_fail (VALA_IS_VARIABLE_DECLARATOR (decl));
+	g_return_if_fail (decl == NULL || VALA_IS_VARIABLE_DECLARATOR (decl));
 	gee_collection_add (GEE_COLLECTION (self->priv->local_variables), decl);
 }
 
@@ -108,7 +108,7 @@ GeeCollection* vala_block_get_local_variables (ValaBlock* self) {
 static void vala_block_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor) {
 	ValaBlock * self;
 	self = VALA_BLOCK (base);
-	g_return_if_fail (VALA_IS_CODE_VISITOR (visitor));
+	g_return_if_fail (visitor == NULL || VALA_IS_CODE_VISITOR (visitor));
 	vala_code_visitor_visit_block (visitor, self);
 }
 
@@ -116,7 +116,7 @@ static void vala_block_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor
 static void vala_block_real_accept_children (ValaCodeNode* base, ValaCodeVisitor* visitor) {
 	ValaBlock * self;
 	self = VALA_BLOCK (base);
-	g_return_if_fail (VALA_IS_CODE_VISITOR (visitor));
+	g_return_if_fail (visitor == NULL || VALA_IS_CODE_VISITOR (visitor));
 	{
 		GeeList* stmt_collection;
 		GeeIterator* stmt_it;
@@ -127,7 +127,7 @@ static void vala_block_real_accept_children (ValaCodeNode* base, ValaCodeVisitor
 			stmt = gee_iterator_get (stmt_it);
 			{
 				vala_code_node_accept (VALA_CODE_NODE (stmt), visitor);
-				(stmt = (g_object_unref (stmt), NULL));
+				(stmt == NULL ? NULL : (stmt = (g_object_unref (stmt), NULL)));
 			}
 		}
 		(stmt_it == NULL ? NULL : (stmt_it = (g_object_unref (stmt_it), NULL)));
