@@ -51,7 +51,6 @@ ValaCharacterLiteral* vala_character_literal_new (const char* c, ValaSourceRefer
 	GParameter * __params;
 	GParameter * __params_it;
 	ValaCharacterLiteral * self;
-	g_return_val_if_fail (c != NULL, NULL);
 	g_return_val_if_fail (source == NULL || VALA_IS_SOURCE_REFERENCE (source), NULL);
 	__params = g_new0 (GParameter, 1);
 	__params_it = __params;
@@ -70,7 +69,7 @@ ValaCharacterLiteral* vala_character_literal_new (const char* c, ValaSourceRefer
 static void vala_character_literal_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor) {
 	ValaCharacterLiteral * self;
 	self = VALA_CHARACTER_LITERAL (base);
-	g_return_if_fail (VALA_IS_CODE_VISITOR (visitor));
+	g_return_if_fail (visitor == NULL || VALA_IS_CODE_VISITOR (visitor));
 	vala_code_visitor_visit_character_literal (visitor, self);
 }
 
@@ -94,10 +93,12 @@ char* vala_character_literal_get_value (ValaCharacterLiteral* self) {
 
 
 void vala_character_literal_set_value (ValaCharacterLiteral* self, const char* value) {
-	char* _tmp1;
+	char* _tmp2;
+	const char* _tmp1;
 	g_return_if_fail (VALA_IS_CHARACTER_LITERAL (self));
+	_tmp2 = NULL;
 	_tmp1 = NULL;
-	self->priv->_value = (_tmp1 = g_strdup (value), (self->priv->_value = (g_free (self->priv->_value), NULL)), _tmp1);
+	self->priv->_value = (_tmp2 = (_tmp1 = value, (_tmp1 == NULL ? NULL : g_strdup (_tmp1))), (self->priv->_value = (g_free (self->priv->_value), NULL)), _tmp2);
 	if (!g_utf8_validate (value, -1, NULL) || (g_utf8_strlen (value, -1) != 3 && g_utf8_get_char (g_utf8_next_char (value)) != '\\')) {
 		vala_code_node_set_error (VALA_CODE_NODE (self), TRUE);
 	}
