@@ -1,6 +1,6 @@
 /* valaclass.vala
  *
- * Copyright (C) 2006-2007  Jürg Billeter
+ * Copyright (C) 2006-2008  Jürg Billeter
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -25,7 +25,7 @@
 #include <gee/list.h>
 #include <gee/readonlycollection.h>
 #include <vala/valatypeparameter.h>
-#include <vala/valatypereference.h>
+#include <vala/valadatatype.h>
 #include <vala/valaconstant.h>
 #include <vala/valafield.h>
 #include <vala/valamethod.h>
@@ -39,9 +39,12 @@
 #include <vala/valascope.h>
 #include <vala/valacreationmethod.h>
 #include <vala/valaformalparameter.h>
+#include <vala/valaclasstype.h>
+#include <vala/valavoidtype.h>
+#include <vala/valavariabledeclarator.h>
+#include <vala/valaexpression.h>
 #include <vala/valapropertyaccessor.h>
 #include <vala/valasourcefile.h>
-#include <vala/valaexpression.h>
 #include <vala/valacodevisitor.h>
 #include <vala/valaattribute.h>
 
@@ -90,25 +93,26 @@ enum  {
 static void vala_class_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor);
 static void vala_class_real_accept_children (ValaCodeNode* base, ValaCodeVisitor* visitor);
 static char* vala_class_real_get_cprefix (ValaSymbol* base);
-static char* vala_class_real_get_cname (ValaDataType* base, gboolean const_type);
+static char* vala_class_real_get_cname (ValaTypesymbol* base, gboolean const_type);
 static char* vala_class_get_lower_case_csuffix (ValaClass* self);
 static char* vala_class_real_get_lower_case_cname (ValaSymbol* base, const char* infix);
 static char* vala_class_real_get_lower_case_cprefix (ValaSymbol* base);
-static char* vala_class_real_get_upper_case_cname (ValaDataType* base, const char* infix);
-static gboolean vala_class_real_is_reference_type (ValaDataType* base);
+static char* vala_class_real_get_upper_case_cname (ValaTypesymbol* base, const char* infix);
+static gboolean vala_class_real_is_reference_type (ValaTypesymbol* base);
 static void vala_class_process_ccode_attribute (ValaClass* self, ValaAttribute* a);
 static gboolean vala_class_get_has_type_id (ValaClass* self);
-static char* vala_class_real_get_type_id (ValaDataType* base);
-static char* vala_class_real_get_marshaller_type_name (ValaDataType* base);
-static char* vala_class_real_get_get_value_function (ValaDataType* base);
-static char* vala_class_real_get_set_value_function (ValaDataType* base);
-static gboolean vala_class_real_is_reference_counting (ValaDataType* base);
-static char* vala_class_real_get_ref_function (ValaDataType* base);
-static char* vala_class_real_get_unref_function (ValaDataType* base);
-static char* vala_class_real_get_dup_function (ValaDataType* base);
-static char* vala_class_real_get_free_function (ValaDataType* base);
-static gboolean vala_class_real_is_subtype_of (ValaDataType* base, ValaDataType* t);
-static gint vala_class_real_get_type_parameter_index (ValaDataType* base, const char* name);
+static char* vala_class_real_get_type_id (ValaTypesymbol* base);
+static char* vala_class_real_get_marshaller_type_name (ValaTypesymbol* base);
+static char* vala_class_real_get_get_value_function (ValaTypesymbol* base);
+static char* vala_class_real_get_set_value_function (ValaTypesymbol* base);
+static gboolean vala_class_real_is_reference_counting (ValaTypesymbol* base);
+static char* vala_class_real_get_ref_function (ValaTypesymbol* base);
+static char* vala_class_real_get_unref_function (ValaTypesymbol* base);
+static char* vala_class_real_get_dup_function (ValaTypesymbol* base);
+static char* vala_class_real_get_free_function (ValaTypesymbol* base);
+static gboolean vala_class_real_is_subtype_of (ValaTypesymbol* base, ValaTypesymbol* t);
+static gint vala_class_real_get_type_parameter_index (ValaTypesymbol* base, const char* name);
+static void vala_class_real_replace_type (ValaCodeNode* base, ValaDataType* old_type, ValaDataType* new_type);
 static gpointer vala_class_parent_class = NULL;
 static void vala_class_dispose (GObject * obj);
 static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func);
@@ -123,7 +127,6 @@ static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify
  */
 ValaClass* vala_class_new (const char* name, ValaSourceReference* source_reference) {
 	ValaClass * self;
-	g_return_val_if_fail (name != NULL, NULL);
 	g_return_val_if_fail (source_reference == NULL || VALA_IS_SOURCE_REFERENCE (source_reference), NULL);
 	self = g_object_newv (VALA_TYPE_CLASS, 0, NULL);
 	vala_symbol_set_name (VALA_SYMBOL (self), name);
@@ -138,10 +141,11 @@ ValaClass* vala_class_new (const char* name, ValaSourceReference* source_referen
  *
  * @param type a class or interface reference
  */
-void vala_class_add_base_type (ValaClass* self, ValaTypeReference* type) {
+void vala_class_add_base_type (ValaClass* self, ValaDataType* type) {
 	g_return_if_fail (VALA_IS_CLASS (self));
-	g_return_if_fail (VALA_IS_TYPE_REFERENCE (type));
+	g_return_if_fail (type == NULL || VALA_IS_DATA_TYPE (type));
 	gee_collection_add (GEE_COLLECTION (self->priv->base_types), type);
+	vala_code_node_set_parent_node (VALA_CODE_NODE (type), VALA_CODE_NODE (self));
 }
 
 
@@ -152,7 +156,7 @@ void vala_class_add_base_type (ValaClass* self, ValaTypeReference* type) {
  */
 GeeCollection* vala_class_get_base_types (ValaClass* self) {
 	g_return_val_if_fail (VALA_IS_CLASS (self), NULL);
-	return GEE_COLLECTION (gee_read_only_collection_new (VALA_TYPE_TYPE_REFERENCE, ((GBoxedCopyFunc) g_object_ref), g_object_unref, GEE_COLLECTION (self->priv->base_types)));
+	return GEE_COLLECTION (gee_read_only_collection_new (VALA_TYPE_DATA_TYPE, ((GBoxedCopyFunc) g_object_ref), g_object_unref, GEE_COLLECTION (self->priv->base_types)));
 }
 
 
@@ -163,9 +167,9 @@ GeeCollection* vala_class_get_base_types (ValaClass* self) {
  */
 void vala_class_add_type_parameter (ValaClass* self, ValaTypeParameter* p) {
 	g_return_if_fail (VALA_IS_CLASS (self));
-	g_return_if_fail (VALA_IS_TYPE_PARAMETER (p));
+	g_return_if_fail (p == NULL || VALA_IS_TYPEPARAMETER (p));
 	gee_collection_add (GEE_COLLECTION (self->priv->type_parameters), p);
-	p->type = VALA_DATA_TYPE (self);
+	p->type = VALA_TYPESYMBOL (self);
 	vala_scope_add (vala_symbol_get_scope (VALA_SYMBOL (self)), vala_symbol_get_name (VALA_SYMBOL (p)), VALA_SYMBOL (p));
 }
 
@@ -177,7 +181,7 @@ void vala_class_add_type_parameter (ValaClass* self, ValaTypeParameter* p) {
  */
 GeeCollection* vala_class_get_type_parameters (ValaClass* self) {
 	g_return_val_if_fail (VALA_IS_CLASS (self), NULL);
-	return GEE_COLLECTION (gee_read_only_collection_new (VALA_TYPE_TYPE_PARAMETER, ((GBoxedCopyFunc) g_object_ref), g_object_unref, GEE_COLLECTION (self->priv->type_parameters)));
+	return GEE_COLLECTION (gee_read_only_collection_new (VALA_TYPE_TYPEPARAMETER, ((GBoxedCopyFunc) g_object_ref), g_object_unref, GEE_COLLECTION (self->priv->type_parameters)));
 }
 
 
@@ -188,7 +192,7 @@ GeeCollection* vala_class_get_type_parameters (ValaClass* self) {
  */
 void vala_class_add_constant (ValaClass* self, ValaConstant* c) {
 	g_return_if_fail (VALA_IS_CLASS (self));
-	g_return_if_fail (VALA_IS_CONSTANT (c));
+	g_return_if_fail (c == NULL || VALA_IS_CONSTANT (c));
 	gee_collection_add (GEE_COLLECTION (self->priv->constants), c);
 	vala_scope_add (vala_symbol_get_scope (VALA_SYMBOL (self)), vala_symbol_get_name (VALA_SYMBOL (c)), VALA_SYMBOL (c));
 }
@@ -201,9 +205,7 @@ void vala_class_add_constant (ValaClass* self, ValaConstant* c) {
  */
 void vala_class_add_field (ValaClass* self, ValaField* f) {
 	g_return_if_fail (VALA_IS_CLASS (self));
-	g_return_if_fail (VALA_IS_FIELD (f));
-	/* non_null fields not yet supported due to initialization issues*/
-	vala_type_reference_set_non_null (vala_field_get_type_reference (f), FALSE);
+	g_return_if_fail (f == NULL || VALA_IS_FIELD (f));
 	gee_collection_add (GEE_COLLECTION (self->priv->fields), f);
 	if (vala_symbol_get_access (VALA_SYMBOL (f)) == VALA_SYMBOL_ACCESSIBILITY_PRIVATE && vala_field_get_instance (f)) {
 		self->priv->_has_private_fields = TRUE;
@@ -230,20 +232,38 @@ GeeCollection* vala_class_get_fields (ValaClass* self) {
  */
 void vala_class_add_method (ValaClass* self, ValaMethod* m) {
 	g_return_if_fail (VALA_IS_CLASS (self));
-	g_return_if_fail (VALA_IS_METHOD (m));
+	g_return_if_fail (m == NULL || VALA_IS_METHOD (m));
 	if (vala_method_get_instance (m) || VALA_IS_CREATION_METHOD (m)) {
 		ValaFormalParameter* _tmp1;
-		ValaTypeReference* _tmp0;
+		ValaClassType* _tmp0;
 		_tmp1 = NULL;
 		_tmp0 = NULL;
-		vala_method_set_this_parameter (m, (_tmp1 = vala_formal_parameter_new ("this", (_tmp0 = vala_type_reference_new ()), NULL)));
+		vala_method_set_this_parameter (m, (_tmp1 = vala_formal_parameter_new ("this", VALA_DATA_TYPE ((_tmp0 = vala_class_type_new (self))), NULL)));
 		(_tmp1 == NULL ? NULL : (_tmp1 = (g_object_unref (_tmp1), NULL)));
 		(_tmp0 == NULL ? NULL : (_tmp0 = (g_object_unref (_tmp0), NULL)));
-		vala_type_reference_set_data_type (vala_formal_parameter_get_type_reference (vala_method_get_this_parameter (m)), VALA_DATA_TYPE (self));
 		vala_scope_add (vala_symbol_get_scope (VALA_SYMBOL (m)), vala_symbol_get_name (VALA_SYMBOL (vala_method_get_this_parameter (m))), VALA_SYMBOL (vala_method_get_this_parameter (m)));
 	}
-	if (VALA_IS_CREATION_METHOD (m) && vala_symbol_get_name (VALA_SYMBOL (m)) == NULL) {
-		vala_class_set_default_construction_method (self, m);
+	if (!(VALA_IS_VOID_TYPE (vala_method_get_return_type (m)))) {
+		ValaVariableDeclarator* _tmp2;
+		ValaDataType* _tmp3;
+		_tmp2 = NULL;
+		vala_method_set_result_var (m, (_tmp2 = vala_variable_declarator_new ("result", NULL, NULL)));
+		(_tmp2 == NULL ? NULL : (_tmp2 = (g_object_unref (_tmp2), NULL)));
+		_tmp3 = NULL;
+		vala_variable_declarator_set_type_reference (vala_method_get_result_var (m), (_tmp3 = vala_data_type_copy (vala_method_get_return_type (m))));
+		(_tmp3 == NULL ? NULL : (_tmp3 = (g_object_unref (_tmp3), NULL)));
+		vala_scope_add (vala_symbol_get_scope (VALA_SYMBOL (m)), vala_symbol_get_name (VALA_SYMBOL (vala_method_get_result_var (m))), VALA_SYMBOL (vala_method_get_result_var (m)));
+	}
+	if (VALA_IS_CREATION_METHOD (m)) {
+		if (vala_symbol_get_name (VALA_SYMBOL (m)) == NULL) {
+			vala_class_set_default_construction_method (self, m);
+			vala_symbol_set_name (VALA_SYMBOL (m), ".new");
+		} else {
+			char* _tmp4;
+			_tmp4 = NULL;
+			vala_symbol_set_name (VALA_SYMBOL (m), (_tmp4 = g_strconcat (".new.", vala_symbol_get_name (VALA_SYMBOL (m)), NULL)));
+			(_tmp4 = (g_free (_tmp4), NULL));
+		}
 	}
 	gee_collection_add (GEE_COLLECTION (self->priv->methods), m);
 	vala_scope_add (vala_symbol_get_scope (VALA_SYMBOL (self)), vala_symbol_get_name (VALA_SYMBOL (m)), VALA_SYMBOL (m));
@@ -268,33 +288,30 @@ GeeCollection* vala_class_get_methods (ValaClass* self) {
  */
 void vala_class_add_property (ValaClass* self, ValaProperty* prop, gboolean no_field) {
 	ValaFormalParameter* _tmp1;
-	ValaTypeReference* _tmp0;
+	ValaClassType* _tmp0;
 	g_return_if_fail (VALA_IS_CLASS (self));
-	g_return_if_fail (VALA_IS_PROPERTY (prop));
+	g_return_if_fail (prop == NULL || VALA_IS_PROPERTY (prop));
 	gee_collection_add (GEE_COLLECTION (self->priv->properties), prop);
 	vala_scope_add (vala_symbol_get_scope (VALA_SYMBOL (self)), vala_symbol_get_name (VALA_SYMBOL (prop)), VALA_SYMBOL (prop));
 	_tmp1 = NULL;
 	_tmp0 = NULL;
-	vala_property_set_this_parameter (prop, (_tmp1 = vala_formal_parameter_new ("this", (_tmp0 = vala_type_reference_new ()), NULL)));
+	vala_property_set_this_parameter (prop, (_tmp1 = vala_formal_parameter_new ("this", VALA_DATA_TYPE ((_tmp0 = vala_class_type_new (self))), NULL)));
 	(_tmp1 == NULL ? NULL : (_tmp1 = (g_object_unref (_tmp1), NULL)));
 	(_tmp0 == NULL ? NULL : (_tmp0 = (g_object_unref (_tmp0), NULL)));
-	vala_type_reference_set_data_type (vala_formal_parameter_get_type_reference (vala_property_get_this_parameter (prop)), VALA_DATA_TYPE (self));
 	vala_scope_add (vala_symbol_get_scope (VALA_SYMBOL (prop)), vala_symbol_get_name (VALA_SYMBOL (vala_property_get_this_parameter (prop))), VALA_SYMBOL (vala_property_get_this_parameter (prop)));
 	if (!no_field && vala_property_get_set_accessor (prop) != NULL && vala_property_accessor_get_body (vala_property_get_set_accessor (prop)) == NULL && vala_code_node_get_source_reference (VALA_CODE_NODE (self)) != NULL && !vala_source_file_get_pkg (vala_source_reference_get_file (vala_code_node_get_source_reference (VALA_CODE_NODE (self))))) {
-		ValaTypeReference* field_type;
+		ValaDataType* field_type;
 		char* _tmp2;
 		ValaField* _tmp3;
 		ValaField* f;
 		/* automatic property accessor body generation */
-		field_type = vala_type_reference_copy (vala_property_get_type_reference (prop));
-		/* non_null fields not yet supported due to initialization issues*/
-		vala_type_reference_set_non_null (field_type, FALSE);
+		field_type = vala_data_type_copy (vala_property_get_type_reference (prop));
 		_tmp2 = NULL;
 		_tmp3 = NULL;
 		f = (_tmp3 = vala_field_new ((_tmp2 = g_strdup_printf ("_%s", vala_symbol_get_name (VALA_SYMBOL (prop)))), field_type, NULL, vala_code_node_get_source_reference (VALA_CODE_NODE (prop))), (_tmp2 = (g_free (_tmp2), NULL)), _tmp3);
 		vala_symbol_set_access (VALA_SYMBOL (f), VALA_SYMBOL_ACCESSIBILITY_PRIVATE);
 		vala_class_add_field (self, f);
-		(field_type = (g_object_unref (field_type), NULL));
+		(field_type == NULL ? NULL : (field_type = (g_object_unref (field_type), NULL)));
 		(f == NULL ? NULL : (f = (g_object_unref (f), NULL)));
 	}
 }
@@ -318,7 +335,7 @@ GeeCollection* vala_class_get_properties (ValaClass* self) {
  */
 void vala_class_add_signal (ValaClass* self, ValaSignal* sig) {
 	g_return_if_fail (VALA_IS_CLASS (self));
-	g_return_if_fail (VALA_IS_SIGNAL (sig));
+	g_return_if_fail (sig == NULL || VALA_IS_SIGNAL (sig));
 	gee_collection_add (GEE_COLLECTION (self->priv->signals), sig);
 	vala_scope_add (vala_symbol_get_scope (VALA_SYMBOL (self)), vala_symbol_get_name (VALA_SYMBOL (sig)), VALA_SYMBOL (sig));
 }
@@ -342,7 +359,7 @@ GeeCollection* vala_class_get_signals (ValaClass* self) {
  */
 void vala_class_add_class (ValaClass* self, ValaClass* cl) {
 	g_return_if_fail (VALA_IS_CLASS (self));
-	g_return_if_fail (VALA_IS_CLASS (cl));
+	g_return_if_fail (cl == NULL || VALA_IS_CLASS (cl));
 	gee_collection_add (GEE_COLLECTION (self->priv->classes), cl);
 	vala_scope_add (vala_symbol_get_scope (VALA_SYMBOL (self)), vala_symbol_get_name (VALA_SYMBOL (cl)), VALA_SYMBOL (cl));
 }
@@ -355,7 +372,7 @@ void vala_class_add_class (ValaClass* self, ValaClass* cl) {
  */
 void vala_class_add_struct (ValaClass* self, ValaStruct* st) {
 	g_return_if_fail (VALA_IS_CLASS (self));
-	g_return_if_fail (VALA_IS_STRUCT (st));
+	g_return_if_fail (st == NULL || VALA_IS_STRUCT (st));
 	gee_collection_add (GEE_COLLECTION (self->priv->structs), st);
 	vala_scope_add (vala_symbol_get_scope (VALA_SYMBOL (self)), vala_symbol_get_name (VALA_SYMBOL (st)), VALA_SYMBOL (st));
 }
@@ -364,7 +381,7 @@ void vala_class_add_struct (ValaClass* self, ValaStruct* st) {
 static void vala_class_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor) {
 	ValaClass * self;
 	self = VALA_CLASS (base);
-	g_return_if_fail (VALA_IS_CODE_VISITOR (visitor));
+	g_return_if_fail (visitor == NULL || VALA_IS_CODE_VISITOR (visitor));
 	vala_code_visitor_visit_class (visitor, self);
 }
 
@@ -372,14 +389,14 @@ static void vala_class_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor
 static void vala_class_real_accept_children (ValaCodeNode* base, ValaCodeVisitor* visitor) {
 	ValaClass * self;
 	self = VALA_CLASS (base);
-	g_return_if_fail (VALA_IS_CODE_VISITOR (visitor));
+	g_return_if_fail (visitor == NULL || VALA_IS_CODE_VISITOR (visitor));
 	{
 		GeeList* type_collection;
 		GeeIterator* type_it;
 		type_collection = self->priv->base_types;
 		type_it = gee_iterable_iterator (GEE_ITERABLE (type_collection));
 		while (gee_iterator_next (type_it)) {
-			ValaTypeReference* type;
+			ValaDataType* type;
 			type = gee_iterator_get (type_it);
 			{
 				vala_code_node_accept (VALA_CODE_NODE (type), visitor);
@@ -520,11 +537,11 @@ static void vala_class_real_accept_children (ValaCodeNode* base, ValaCodeVisitor
 static char* vala_class_real_get_cprefix (ValaSymbol* base) {
 	ValaClass * self;
 	self = VALA_CLASS (base);
-	return vala_data_type_get_cname (VALA_DATA_TYPE (self), FALSE);
+	return vala_typesymbol_get_cname (VALA_TYPESYMBOL (self), FALSE);
 }
 
 
-static char* vala_class_real_get_cname (ValaDataType* base, gboolean const_type) {
+static char* vala_class_real_get_cname (ValaTypesymbol* base, gboolean const_type) {
 	ValaClass * self;
 	const char* _tmp3;
 	self = VALA_CLASS (base);
@@ -564,24 +581,47 @@ char* vala_class_get_default_cname (ValaClass* self) {
  * @param cname the name to be used in C code
  */
 void vala_class_set_cname (ValaClass* self, const char* cname) {
-	char* _tmp0;
+	char* _tmp1;
+	const char* _tmp0;
 	g_return_if_fail (VALA_IS_CLASS (self));
-	g_return_if_fail (cname != NULL);
+	_tmp1 = NULL;
 	_tmp0 = NULL;
-	self->priv->cname = (_tmp0 = g_strdup (cname), (self->priv->cname = (g_free (self->priv->cname), NULL)), _tmp0);
+	self->priv->cname = (_tmp1 = (_tmp0 = cname, (_tmp0 == NULL ? NULL : g_strdup (_tmp0))), (self->priv->cname = (g_free (self->priv->cname), NULL)), _tmp1);
 }
 
 
 static char* vala_class_get_lower_case_csuffix (ValaClass* self) {
-	const char* _tmp1;
+	const char* _tmp6;
 	g_return_val_if_fail (VALA_IS_CLASS (self), NULL);
 	if (self->priv->lower_case_csuffix == NULL) {
 		char* _tmp0;
 		_tmp0 = NULL;
 		self->priv->lower_case_csuffix = (_tmp0 = vala_symbol_camel_case_to_lower_case (vala_symbol_get_name (VALA_SYMBOL (self))), (self->priv->lower_case_csuffix = (g_free (self->priv->lower_case_csuffix), NULL)), _tmp0);
+		/* remove underscores in some cases to avoid conflicts of type macros*/
+		if (g_str_has_prefix (self->priv->lower_case_csuffix, "type_")) {
+			char* _tmp1;
+			_tmp1 = NULL;
+			self->priv->lower_case_csuffix = (_tmp1 = g_strconcat ("type", g_utf8_offset_to_pointer (self->priv->lower_case_csuffix, g_utf8_strlen ("type_", -1)), NULL), (self->priv->lower_case_csuffix = (g_free (self->priv->lower_case_csuffix), NULL)), _tmp1);
+		} else {
+			if (g_str_has_prefix (self->priv->lower_case_csuffix, "is_")) {
+				char* _tmp2;
+				_tmp2 = NULL;
+				self->priv->lower_case_csuffix = (_tmp2 = g_strconcat ("is", g_utf8_offset_to_pointer (self->priv->lower_case_csuffix, g_utf8_strlen ("is_", -1)), NULL), (self->priv->lower_case_csuffix = (g_free (self->priv->lower_case_csuffix), NULL)), _tmp2);
+			}
+		}
+		if (g_str_has_suffix (self->priv->lower_case_csuffix, "_class")) {
+			char* _tmp5;
+			char* _tmp4;
+			char* _tmp3;
+			_tmp5 = NULL;
+			_tmp4 = NULL;
+			_tmp3 = NULL;
+			self->priv->lower_case_csuffix = (_tmp5 = g_strconcat ((_tmp4 = g_utf8_offset_to_pointer (self->priv->lower_case_csuffix, 0), g_strndup (_tmp4, g_utf8_offset_to_pointer (_tmp4, g_utf8_strlen (self->priv->lower_case_csuffix, -1) - g_utf8_strlen ("_class", -1)) - _tmp4)), "class", NULL), (self->priv->lower_case_csuffix = (g_free (self->priv->lower_case_csuffix), NULL)), _tmp5);
+			(_tmp3 = (g_free (_tmp3), NULL));
+		}
 	}
-	_tmp1 = NULL;
-	return (_tmp1 = self->priv->lower_case_csuffix, (_tmp1 == NULL ? NULL : g_strdup (_tmp1)));
+	_tmp6 = NULL;
+	return (_tmp6 = self->priv->lower_case_csuffix, (_tmp6 == NULL ? NULL : g_strdup (_tmp6)));
 }
 
 
@@ -618,7 +658,7 @@ static char* vala_class_real_get_lower_case_cprefix (ValaSymbol* base) {
 }
 
 
-static char* vala_class_real_get_upper_case_cname (ValaDataType* base, const char* infix) {
+static char* vala_class_real_get_upper_case_cname (ValaTypesymbol* base, const char* infix) {
 	ValaClass * self;
 	char* _tmp0;
 	char* _tmp1;
@@ -629,7 +669,7 @@ static char* vala_class_real_get_upper_case_cname (ValaDataType* base, const cha
 }
 
 
-static gboolean vala_class_real_is_reference_type (ValaDataType* base) {
+static gboolean vala_class_real_is_reference_type (ValaTypesymbol* base) {
 	ValaClass * self;
 	self = VALA_CLASS (base);
 	return TRUE;
@@ -638,7 +678,7 @@ static gboolean vala_class_real_is_reference_type (ValaDataType* base) {
 
 static void vala_class_process_ccode_attribute (ValaClass* self, ValaAttribute* a) {
 	g_return_if_fail (VALA_IS_CLASS (self));
-	g_return_if_fail (VALA_IS_ATTRIBUTE (a));
+	g_return_if_fail (a == NULL || VALA_IS_ATTRIBUTE (a));
 	if (vala_attribute_has_argument (a, "ref_function")) {
 		char* _tmp0;
 		_tmp0 = NULL;
@@ -722,7 +762,7 @@ static void vala_class_process_ccode_attribute (ValaClass* self, ValaAttribute*
 				_tmp12 = NULL;
 				filename = (_tmp12 = *filename_it, (_tmp12 == NULL ? NULL : g_strdup (_tmp12)));
 				{
-					vala_data_type_add_cheader_filename (VALA_DATA_TYPE (self), filename);
+					vala_typesymbol_add_cheader_filename (VALA_TYPESYMBOL (self), filename);
 					(filename = (g_free (filename), NULL));
 				}
 			}
@@ -764,7 +804,7 @@ static gboolean vala_class_get_has_type_id (ValaClass* self) {
 }
 
 
-static char* vala_class_real_get_type_id (ValaDataType* base) {
+static char* vala_class_real_get_type_id (ValaTypesymbol* base) {
 	ValaClass * self;
 	const char* _tmp2;
 	self = VALA_CLASS (base);
@@ -772,7 +812,7 @@ static char* vala_class_real_get_type_id (ValaDataType* base) {
 		if (vala_class_get_has_type_id (self)) {
 			char* _tmp0;
 			_tmp0 = NULL;
-			self->priv->type_id = (_tmp0 = vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (self), "TYPE_"), (self->priv->type_id = (g_free (self->priv->type_id), NULL)), _tmp0);
+			self->priv->type_id = (_tmp0 = vala_typesymbol_get_upper_case_cname (VALA_TYPESYMBOL (self), "TYPE_"), (self->priv->type_id = (g_free (self->priv->type_id), NULL)), _tmp0);
 		} else {
 			char* _tmp1;
 			_tmp1 = NULL;
@@ -785,15 +825,16 @@ static char* vala_class_real_get_type_id (ValaDataType* base) {
 
 
 void vala_class_set_type_id (ValaClass* self, const char* type_id) {
-	char* _tmp0;
+	char* _tmp1;
+	const char* _tmp0;
 	g_return_if_fail (VALA_IS_CLASS (self));
-	g_return_if_fail (type_id != NULL);
+	_tmp1 = NULL;
 	_tmp0 = NULL;
-	self->priv->type_id = (_tmp0 = g_strdup (type_id), (self->priv->type_id = (g_free (self->priv->type_id), NULL)), _tmp0);
+	self->priv->type_id = (_tmp1 = (_tmp0 = type_id, (_tmp0 == NULL ? NULL : g_strdup (_tmp0))), (self->priv->type_id = (g_free (self->priv->type_id), NULL)), _tmp1);
 }
 
 
-static char* vala_class_real_get_marshaller_type_name (ValaDataType* base) {
+static char* vala_class_real_get_marshaller_type_name (ValaTypesymbol* base) {
 	ValaClass * self;
 	const char* _tmp2;
 	self = VALA_CLASS (base);
@@ -801,7 +842,7 @@ static char* vala_class_real_get_marshaller_type_name (ValaDataType* base) {
 		if (vala_class_get_base_class (self) != NULL) {
 			char* _tmp0;
 			_tmp0 = NULL;
-			self->priv->marshaller_type_name = (_tmp0 = vala_data_type_get_marshaller_type_name (VALA_DATA_TYPE (vala_class_get_base_class (self))), (self->priv->marshaller_type_name = (g_free (self->priv->marshaller_type_name), NULL)), _tmp0);
+			self->priv->marshaller_type_name = (_tmp0 = vala_typesymbol_get_marshaller_type_name (VALA_TYPESYMBOL (vala_class_get_base_class (self))), (self->priv->marshaller_type_name = (g_free (self->priv->marshaller_type_name), NULL)), _tmp0);
 		} else {
 			char* _tmp1;
 			_tmp1 = NULL;
@@ -813,7 +854,7 @@ static char* vala_class_real_get_marshaller_type_name (ValaDataType* base) {
 }
 
 
-static char* vala_class_real_get_get_value_function (ValaDataType* base) {
+static char* vala_class_real_get_get_value_function (ValaTypesymbol* base) {
 	ValaClass * self;
 	const char* _tmp2;
 	self = VALA_CLASS (base);
@@ -821,7 +862,7 @@ static char* vala_class_real_get_get_value_function (ValaDataType* base) {
 		if (vala_class_get_base_class (self) != NULL) {
 			char* _tmp0;
 			_tmp0 = NULL;
-			self->priv->get_value_function = (_tmp0 = vala_data_type_get_get_value_function (VALA_DATA_TYPE (vala_class_get_base_class (self))), (self->priv->get_value_function = (g_free (self->priv->get_value_function), NULL)), _tmp0);
+			self->priv->get_value_function = (_tmp0 = vala_typesymbol_get_get_value_function (VALA_TYPESYMBOL (vala_class_get_base_class (self))), (self->priv->get_value_function = (g_free (self->priv->get_value_function), NULL)), _tmp0);
 		} else {
 			char* _tmp1;
 			_tmp1 = NULL;
@@ -833,7 +874,7 @@ static char* vala_class_real_get_get_value_function (ValaDataType* base) {
 }
 
 
-static char* vala_class_real_get_set_value_function (ValaDataType* base) {
+static char* vala_class_real_get_set_value_function (ValaTypesymbol* base) {
 	ValaClass * self;
 	const char* _tmp2;
 	self = VALA_CLASS (base);
@@ -841,7 +882,7 @@ static char* vala_class_real_get_set_value_function (ValaDataType* base) {
 		if (vala_class_get_base_class (self) != NULL) {
 			char* _tmp0;
 			_tmp0 = NULL;
-			self->priv->set_value_function = (_tmp0 = vala_data_type_get_set_value_function (VALA_DATA_TYPE (vala_class_get_base_class (self))), (self->priv->set_value_function = (g_free (self->priv->set_value_function), NULL)), _tmp0);
+			self->priv->set_value_function = (_tmp0 = vala_typesymbol_get_set_value_function (VALA_TYPESYMBOL (vala_class_get_base_class (self))), (self->priv->set_value_function = (g_free (self->priv->set_value_function), NULL)), _tmp0);
 		} else {
 			char* _tmp1;
 			_tmp1 = NULL;
@@ -853,21 +894,21 @@ static char* vala_class_real_get_set_value_function (ValaDataType* base) {
 }
 
 
-static gboolean vala_class_real_is_reference_counting (ValaDataType* base) {
+static gboolean vala_class_real_is_reference_counting (ValaTypesymbol* base) {
 	ValaClass * self;
 	char* _tmp0;
 	gboolean _tmp1;
 	self = VALA_CLASS (base);
 	_tmp0 = NULL;
-	return (_tmp1 = (_tmp0 = vala_data_type_get_ref_function (VALA_DATA_TYPE (self))) != NULL, (_tmp0 = (g_free (_tmp0), NULL)), _tmp1);
+	return (_tmp1 = (_tmp0 = vala_typesymbol_get_ref_function (VALA_TYPESYMBOL (self))) != NULL, (_tmp0 = (g_free (_tmp0), NULL)), _tmp1);
 }
 
 
-static char* vala_class_real_get_ref_function (ValaDataType* base) {
+static char* vala_class_real_get_ref_function (ValaTypesymbol* base) {
 	ValaClass * self;
 	self = VALA_CLASS (base);
 	if (self->priv->ref_function == NULL && vala_class_get_base_class (self) != NULL) {
-		return vala_data_type_get_ref_function (VALA_DATA_TYPE (vala_class_get_base_class (self)));
+		return vala_typesymbol_get_ref_function (VALA_TYPESYMBOL (vala_class_get_base_class (self)));
 	} else {
 		const char* _tmp1;
 		_tmp1 = NULL;
@@ -877,19 +918,20 @@ static char* vala_class_real_get_ref_function (ValaDataType* base) {
 
 
 void vala_class_set_ref_function (ValaClass* self, const char* name) {
-	char* _tmp0;
+	char* _tmp1;
+	const char* _tmp0;
 	g_return_if_fail (VALA_IS_CLASS (self));
-	g_return_if_fail (name != NULL);
+	_tmp1 = NULL;
 	_tmp0 = NULL;
-	self->priv->ref_function = (_tmp0 = g_strdup (name), (self->priv->ref_function = (g_free (self->priv->ref_function), NULL)), _tmp0);
+	self->priv->ref_function = (_tmp1 = (_tmp0 = name, (_tmp0 == NULL ? NULL : g_strdup (_tmp0))), (self->priv->ref_function = (g_free (self->priv->ref_function), NULL)), _tmp1);
 }
 
 
-static char* vala_class_real_get_unref_function (ValaDataType* base) {
+static char* vala_class_real_get_unref_function (ValaTypesymbol* base) {
 	ValaClass * self;
 	self = VALA_CLASS (base);
 	if (self->priv->unref_function == NULL && vala_class_get_base_class (self) != NULL) {
-		return vala_data_type_get_unref_function (VALA_DATA_TYPE (vala_class_get_base_class (self)));
+		return vala_typesymbol_get_unref_function (VALA_TYPESYMBOL (vala_class_get_base_class (self)));
 	} else {
 		const char* _tmp1;
 		_tmp1 = NULL;
@@ -899,15 +941,16 @@ static char* vala_class_real_get_unref_function (ValaDataType* base) {
 
 
 void vala_class_set_unref_function (ValaClass* self, const char* name) {
-	char* _tmp0;
+	char* _tmp1;
+	const char* _tmp0;
 	g_return_if_fail (VALA_IS_CLASS (self));
-	g_return_if_fail (name != NULL);
+	_tmp1 = NULL;
 	_tmp0 = NULL;
-	self->priv->unref_function = (_tmp0 = g_strdup (name), (self->priv->unref_function = (g_free (self->priv->unref_function), NULL)), _tmp0);
+	self->priv->unref_function = (_tmp1 = (_tmp0 = name, (_tmp0 == NULL ? NULL : g_strdup (_tmp0))), (self->priv->unref_function = (g_free (self->priv->unref_function), NULL)), _tmp1);
 }
 
 
-static char* vala_class_real_get_dup_function (ValaDataType* base) {
+static char* vala_class_real_get_dup_function (ValaTypesymbol* base) {
 	ValaClass * self;
 	const char* _tmp0;
 	self = VALA_CLASS (base);
@@ -917,11 +960,12 @@ static char* vala_class_real_get_dup_function (ValaDataType* base) {
 
 
 void vala_class_set_dup_function (ValaClass* self, const char* name) {
-	char* _tmp0;
+	char* _tmp1;
+	const char* _tmp0;
 	g_return_if_fail (VALA_IS_CLASS (self));
-	g_return_if_fail (name != NULL);
+	_tmp1 = NULL;
 	_tmp0 = NULL;
-	self->priv->copy_function = (_tmp0 = g_strdup (name), (self->priv->copy_function = (g_free (self->priv->copy_function), NULL)), _tmp0);
+	self->priv->copy_function = (_tmp1 = (_tmp0 = name, (_tmp0 == NULL ? NULL : g_strdup (_tmp0))), (self->priv->copy_function = (g_free (self->priv->copy_function), NULL)), _tmp1);
 }
 
 
@@ -935,7 +979,7 @@ char* vala_class_get_default_free_function (ValaClass* self) {
 }
 
 
-static char* vala_class_real_get_free_function (ValaDataType* base) {
+static char* vala_class_real_get_free_function (ValaTypesymbol* base) {
 	ValaClass * self;
 	const char* _tmp1;
 	self = VALA_CLASS (base);
@@ -950,19 +994,20 @@ static char* vala_class_real_get_free_function (ValaDataType* base) {
 
 
 void vala_class_set_free_function (ValaClass* self, const char* name) {
-	char* _tmp0;
+	char* _tmp1;
+	const char* _tmp0;
 	g_return_if_fail (VALA_IS_CLASS (self));
-	g_return_if_fail (name != NULL);
+	_tmp1 = NULL;
 	_tmp0 = NULL;
-	self->priv->free_function = (_tmp0 = g_strdup (name), (self->priv->free_function = (g_free (self->priv->free_function), NULL)), _tmp0);
+	self->priv->free_function = (_tmp1 = (_tmp0 = name, (_tmp0 == NULL ? NULL : g_strdup (_tmp0))), (self->priv->free_function = (g_free (self->priv->free_function), NULL)), _tmp1);
 }
 
 
-static gboolean vala_class_real_is_subtype_of (ValaDataType* base, ValaDataType* t) {
+static gboolean vala_class_real_is_subtype_of (ValaTypesymbol* base, ValaTypesymbol* t) {
 	ValaClass * self;
 	self = VALA_CLASS (base);
-	g_return_val_if_fail (VALA_IS_DATA_TYPE (t), FALSE);
-	if (VALA_DATA_TYPE (self) == t) {
+	g_return_val_if_fail (t == NULL || VALA_IS_TYPESYMBOL (t), FALSE);
+	if (VALA_TYPESYMBOL (self) == t) {
 		return TRUE;
 	}
 	{
@@ -971,10 +1016,10 @@ static gboolean vala_class_real_is_subtype_of (ValaDataType* base, ValaDataType*
 		base_type_collection = self->priv->base_types;
 		base_type_it = gee_iterable_iterator (GEE_ITERABLE (base_type_collection));
 		while (gee_iterator_next (base_type_it)) {
-			ValaTypeReference* base_type;
+			ValaDataType* base_type;
 			base_type = gee_iterator_get (base_type_it);
 			{
-				if (vala_type_reference_get_data_type (base_type) != NULL && vala_data_type_is_subtype_of (vala_type_reference_get_data_type (base_type), t)) {
+				if (vala_data_type_get_data_type (base_type) != NULL && vala_typesymbol_is_subtype_of (vala_data_type_get_data_type (base_type), t)) {
 					gboolean _tmp1;
 					return (_tmp1 = TRUE, (base_type == NULL ? NULL : (base_type = (g_object_unref (base_type), NULL))), (base_type_it == NULL ? NULL : (base_type_it = (g_object_unref (base_type_it), NULL))), _tmp1);
 				}
@@ -987,11 +1032,10 @@ static gboolean vala_class_real_is_subtype_of (ValaDataType* base, ValaDataType*
 }
 
 
-static gint vala_class_real_get_type_parameter_index (ValaDataType* base, const char* name) {
+static gint vala_class_real_get_type_parameter_index (ValaTypesymbol* base, const char* name) {
 	ValaClass * self;
 	gint i;
 	self = VALA_CLASS (base);
-	g_return_val_if_fail (name != NULL, 0);
 	i = 0;
 	{
 		GeeList* parameter_collection;
@@ -1016,6 +1060,27 @@ static gint vala_class_real_get_type_parameter_index (ValaDataType* base, const
 }
 
 
+static void vala_class_real_replace_type (ValaCodeNode* base, ValaDataType* old_type, ValaDataType* new_type) {
+	ValaClass * self;
+	self = VALA_CLASS (base);
+	g_return_if_fail (old_type == NULL || VALA_IS_DATA_TYPE (old_type));
+	g_return_if_fail (new_type == NULL || VALA_IS_DATA_TYPE (new_type));
+	{
+		gint i;
+		i = 0;
+		for (i = 0; i < gee_collection_get_size (GEE_COLLECTION (self->priv->base_types)); i++) {
+			ValaDataType* _tmp0;
+			gboolean _tmp1;
+			_tmp0 = NULL;
+			if ((_tmp1 = (_tmp0 = gee_list_get (((GeeList*) self->priv->base_types), i)) == old_type, (_tmp0 == NULL ? NULL : (_tmp0 = (g_object_unref (_tmp0), NULL))), _tmp1)) {
+				gee_list_set (((GeeList*) self->priv->base_types), i, new_type);
+				return;
+			}
+		}
+	}
+}
+
+
 ValaClass* vala_class_get_base_class (ValaClass* self) {
 	g_return_val_if_fail (VALA_IS_CLASS (self), NULL);
 	return self->priv->_base_class;
@@ -1180,22 +1245,23 @@ static void vala_class_class_init (ValaClassClass * klass) {
 	VALA_CODE_NODE_CLASS (klass)->accept = vala_class_real_accept;
 	VALA_CODE_NODE_CLASS (klass)->accept_children = vala_class_real_accept_children;
 	VALA_SYMBOL_CLASS (klass)->get_cprefix = vala_class_real_get_cprefix;
-	VALA_DATA_TYPE_CLASS (klass)->get_cname = vala_class_real_get_cname;
+	VALA_TYPESYMBOL_CLASS (klass)->get_cname = vala_class_real_get_cname;
 	VALA_SYMBOL_CLASS (klass)->get_lower_case_cname = vala_class_real_get_lower_case_cname;
 	VALA_SYMBOL_CLASS (klass)->get_lower_case_cprefix = vala_class_real_get_lower_case_cprefix;
-	VALA_DATA_TYPE_CLASS (klass)->get_upper_case_cname = vala_class_real_get_upper_case_cname;
-	VALA_DATA_TYPE_CLASS (klass)->is_reference_type = vala_class_real_is_reference_type;
-	VALA_DATA_TYPE_CLASS (klass)->get_type_id = vala_class_real_get_type_id;
-	VALA_DATA_TYPE_CLASS (klass)->get_marshaller_type_name = vala_class_real_get_marshaller_type_name;
-	VALA_DATA_TYPE_CLASS (klass)->get_get_value_function = vala_class_real_get_get_value_function;
-	VALA_DATA_TYPE_CLASS (klass)->get_set_value_function = vala_class_real_get_set_value_function;
-	VALA_DATA_TYPE_CLASS (klass)->is_reference_counting = vala_class_real_is_reference_counting;
-	VALA_DATA_TYPE_CLASS (klass)->get_ref_function = vala_class_real_get_ref_function;
-	VALA_DATA_TYPE_CLASS (klass)->get_unref_function = vala_class_real_get_unref_function;
-	VALA_DATA_TYPE_CLASS (klass)->get_dup_function = vala_class_real_get_dup_function;
-	VALA_DATA_TYPE_CLASS (klass)->get_free_function = vala_class_real_get_free_function;
-	VALA_DATA_TYPE_CLASS (klass)->is_subtype_of = vala_class_real_is_subtype_of;
-	VALA_DATA_TYPE_CLASS (klass)->get_type_parameter_index = vala_class_real_get_type_parameter_index;
+	VALA_TYPESYMBOL_CLASS (klass)->get_upper_case_cname = vala_class_real_get_upper_case_cname;
+	VALA_TYPESYMBOL_CLASS (klass)->is_reference_type = vala_class_real_is_reference_type;
+	VALA_TYPESYMBOL_CLASS (klass)->get_type_id = vala_class_real_get_type_id;
+	VALA_TYPESYMBOL_CLASS (klass)->get_marshaller_type_name = vala_class_real_get_marshaller_type_name;
+	VALA_TYPESYMBOL_CLASS (klass)->get_get_value_function = vala_class_real_get_get_value_function;
+	VALA_TYPESYMBOL_CLASS (klass)->get_set_value_function = vala_class_real_get_set_value_function;
+	VALA_TYPESYMBOL_CLASS (klass)->is_reference_counting = vala_class_real_is_reference_counting;
+	VALA_TYPESYMBOL_CLASS (klass)->get_ref_function = vala_class_real_get_ref_function;
+	VALA_TYPESYMBOL_CLASS (klass)->get_unref_function = vala_class_real_get_unref_function;
+	VALA_TYPESYMBOL_CLASS (klass)->get_dup_function = vala_class_real_get_dup_function;
+	VALA_TYPESYMBOL_CLASS (klass)->get_free_function = vala_class_real_get_free_function;
+	VALA_TYPESYMBOL_CLASS (klass)->is_subtype_of = vala_class_real_is_subtype_of;
+	VALA_TYPESYMBOL_CLASS (klass)->get_type_parameter_index = vala_class_real_get_type_parameter_index;
+	VALA_CODE_NODE_CLASS (klass)->replace_type = vala_class_real_replace_type;
 	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_CLASS_BASE_CLASS, g_param_spec_object ("base-class", "base-class", "base-class", VALA_TYPE_CLASS, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
 	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_CLASS_IS_ABSTRACT, g_param_spec_boolean ("is-abstract", "is-abstract", "is-abstract", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
 	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_CLASS_IS_STATIC, g_param_spec_boolean ("is-static", "is-static", "is-static", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
@@ -1208,8 +1274,8 @@ static void vala_class_class_init (ValaClassClass * klass) {
 
 static void vala_class_init (ValaClass * self) {
 	self->priv = VALA_CLASS_GET_PRIVATE (self);
-	self->priv->type_parameters = GEE_LIST (gee_array_list_new (VALA_TYPE_TYPE_PARAMETER, ((GBoxedCopyFunc) g_object_ref), g_object_unref, g_direct_equal));
-	self->priv->base_types = GEE_LIST (gee_array_list_new (VALA_TYPE_TYPE_REFERENCE, ((GBoxedCopyFunc) g_object_ref), g_object_unref, g_direct_equal));
+	self->priv->type_parameters = GEE_LIST (gee_array_list_new (VALA_TYPE_TYPEPARAMETER, ((GBoxedCopyFunc) g_object_ref), g_object_unref, g_direct_equal));
+	self->priv->base_types = GEE_LIST (gee_array_list_new (VALA_TYPE_DATA_TYPE, ((GBoxedCopyFunc) g_object_ref), g_object_unref, g_direct_equal));
 	self->priv->constants = GEE_LIST (gee_array_list_new (VALA_TYPE_CONSTANT, ((GBoxedCopyFunc) g_object_ref), g_object_unref, g_direct_equal));
 	self->priv->fields = GEE_LIST (gee_array_list_new (VALA_TYPE_FIELD, ((GBoxedCopyFunc) g_object_ref), g_object_unref, g_direct_equal));
 	self->priv->methods = GEE_LIST (gee_array_list_new (VALA_TYPE_METHOD, ((GBoxedCopyFunc) g_object_ref), g_object_unref, g_direct_equal));
@@ -1256,7 +1322,7 @@ GType vala_class_get_type (void) {
 	static GType vala_class_type_id = 0;
 	if (G_UNLIKELY (vala_class_type_id == 0)) {
 		static const GTypeInfo g_define_type_info = { sizeof (ValaClassClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_class_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaClass), 0, (GInstanceInitFunc) vala_class_init };
-		vala_class_type_id = g_type_register_static (VALA_TYPE_DATA_TYPE, "ValaClass", &g_define_type_info, 0);
+		vala_class_type_id = g_type_register_static (VALA_TYPE_TYPESYMBOL, "ValaClass", &g_define_type_info, 0);
 	}
 	return vala_class_type_id;
 }
