@@ -1,6 +1,6 @@
 /* valacodevisitor.vala
  *
- * Copyright (C) 2006-2007  Jürg Billeter, Raffaele Sandrini
+ * Copyright (C) 2006-2008  Jürg Billeter, Raffaele Sandrini
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -44,11 +44,11 @@ typedef struct _ValaCodeVisitorPrivate ValaCodeVisitorPrivate;
  * Abstract code node visitor for traversing source code tree.
  */
 struct _ValaCodeVisitor {
-	GObject parent;
+	GObject parent_instance;
 	ValaCodeVisitorPrivate * priv;
 };
 struct _ValaCodeVisitorClass {
-	GObjectClass parent;
+	GObjectClass parent_class;
 	void (*visit_source_file) (ValaCodeVisitor* self, ValaSourceFile* source_file);
 	void (*visit_namespace) (ValaCodeVisitor* self, ValaNamespace* ns);
 	void (*visit_class) (ValaCodeVisitor* self, ValaClass* cl);
@@ -56,7 +56,7 @@ struct _ValaCodeVisitorClass {
 	void (*visit_interface) (ValaCodeVisitor* self, ValaInterface* iface);
 	void (*visit_enum) (ValaCodeVisitor* self, ValaEnum* en);
 	void (*visit_enum_value) (ValaCodeVisitor* self, ValaEnumValue* ev);
-	void (*visit_callback) (ValaCodeVisitor* self, ValaCallback* cb);
+	void (*visit_delegate) (ValaCodeVisitor* self, ValaDelegate* d);
 	void (*visit_member) (ValaCodeVisitor* self, ValaMember* m);
 	void (*visit_constant) (ValaCodeVisitor* self, ValaConstant* c);
 	void (*visit_field) (ValaCodeVisitor* self, ValaField* f);
@@ -71,7 +71,7 @@ struct _ValaCodeVisitorClass {
 	void (*visit_named_argument) (ValaCodeVisitor* self, ValaNamedArgument* n);
 	void (*visit_type_parameter) (ValaCodeVisitor* self, ValaTypeParameter* p);
 	void (*visit_namespace_reference) (ValaCodeVisitor* self, ValaNamespaceReference* ns);
-	void (*visit_type_reference) (ValaCodeVisitor* self, ValaTypeReference* type);
+	void (*visit_data_type) (ValaCodeVisitor* self, ValaDataType* type);
 	void (*visit_block) (ValaCodeVisitor* self, ValaBlock* b);
 	void (*visit_empty_statement) (ValaCodeVisitor* self, ValaEmptyStatement* stmt);
 	void (*visit_declaration_statement) (ValaCodeVisitor* self, ValaDeclarationStatement* stmt);
@@ -131,7 +131,7 @@ void vala_code_visitor_visit_struct (ValaCodeVisitor* self, ValaStruct* st);
 void vala_code_visitor_visit_interface (ValaCodeVisitor* self, ValaInterface* iface);
 void vala_code_visitor_visit_enum (ValaCodeVisitor* self, ValaEnum* en);
 void vala_code_visitor_visit_enum_value (ValaCodeVisitor* self, ValaEnumValue* ev);
-void vala_code_visitor_visit_callback (ValaCodeVisitor* self, ValaCallback* cb);
+void vala_code_visitor_visit_delegate (ValaCodeVisitor* self, ValaDelegate* d);
 void vala_code_visitor_visit_member (ValaCodeVisitor* self, ValaMember* m);
 void vala_code_visitor_visit_constant (ValaCodeVisitor* self, ValaConstant* c);
 void vala_code_visitor_visit_field (ValaCodeVisitor* self, ValaField* f);
@@ -146,7 +146,7 @@ void vala_code_visitor_visit_destructor (ValaCodeVisitor* self, ValaDestructor*
 void vala_code_visitor_visit_named_argument (ValaCodeVisitor* self, ValaNamedArgument* n);
 void vala_code_visitor_visit_type_parameter (ValaCodeVisitor* self, ValaTypeParameter* p);
 void vala_code_visitor_visit_namespace_reference (ValaCodeVisitor* self, ValaNamespaceReference* ns);
-void vala_code_visitor_visit_type_reference (ValaCodeVisitor* self, ValaTypeReference* type);
+void vala_code_visitor_visit_data_type (ValaCodeVisitor* self, ValaDataType* type);
 void vala_code_visitor_visit_block (ValaCodeVisitor* self, ValaBlock* b);
 void vala_code_visitor_visit_empty_statement (ValaCodeVisitor* self, ValaEmptyStatement* stmt);
 void vala_code_visitor_visit_declaration_statement (ValaCodeVisitor* self, ValaDeclarationStatement* stmt);
