@@ -47,11 +47,11 @@ typedef struct _ValaCreationMethodPrivate ValaCreationMethodPrivate;
  * Represents a type creation method.
  */
 struct _ValaCreationMethod {
-	ValaMethod parent;
+	ValaMethod parent_instance;
 	ValaCreationMethodPrivate * priv;
 };
 struct _ValaCreationMethodClass {
-	ValaMethodClass parent;
+	ValaMethodClass parent_class;
 };
 
 ValaCreationMethod* vala_creation_method_new (const char* type_name, const char* name, ValaSourceReference* source_reference);
