@@ -21,20 +21,20 @@
  */
 
 #include <vala/valaforeachstatement.h>
-#include <vala/valatypereference.h>
 #include <vala/valavariabledeclarator.h>
 #include <vala/valaexpression.h>
+#include <vala/valadatatype.h>
 #include <vala/valasourcereference.h>
 #include <vala/valacodevisitor.h>
 
 struct _ValaForeachStatementPrivate {
-	ValaTypeReference* _type_reference;
 	char* _variable_name;
 	ValaVariableDeclarator* _variable_declarator;
 	ValaVariableDeclarator* _collection_variable_declarator;
 	ValaVariableDeclarator* _iterator_variable_declarator;
 	ValaExpression* _collection;
 	ValaBlock* _body;
+	ValaDataType* _data_type;
 };
 #define VALA_FOREACH_STATEMENT_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_FOREACH_STATEMENT, ValaForeachStatementPrivate))
 enum  {
@@ -49,7 +49,8 @@ enum  {
 };
 static void vala_foreach_statement_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor);
 static void vala_foreach_statement_real_accept_children (ValaCodeNode* base, ValaCodeVisitor* visitor);
-static void vala_foreach_statement_real_replace (ValaCodeNode* base, ValaCodeNode* old_node, ValaCodeNode* new_node);
+static void vala_foreach_statement_real_replace_expression (ValaCodeNode* base, ValaExpression* old_node, ValaExpression* new_node);
+static void vala_foreach_statement_real_replace_type (ValaCodeNode* base, ValaDataType* old_type, ValaDataType* new_type);
 static gpointer vala_foreach_statement_parent_class = NULL;
 static void vala_foreach_statement_dispose (GObject * obj);
 
@@ -63,21 +64,20 @@ static void vala_foreach_statement_dispose (GObject * obj);
  * @param source reference to source code
  * @return       newly created foreach statement
  */
-ValaForeachStatement* vala_foreach_statement_new (ValaTypeReference* type_reference, const char* variable_name, ValaExpression* collection, ValaBlock* body, ValaSourceReference* source_reference) {
+ValaForeachStatement* vala_foreach_statement_new (ValaDataType* type_reference, const char* variable_name, ValaExpression* collection, ValaBlock* body, ValaSourceReference* source_reference) {
 	GParameter * __params;
 	GParameter * __params_it;
 	ValaForeachStatement * self;
-	g_return_val_if_fail (VALA_IS_TYPE_REFERENCE (type_reference), NULL);
-	g_return_val_if_fail (variable_name != NULL, NULL);
-	g_return_val_if_fail (VALA_IS_EXPRESSION (collection), NULL);
+	g_return_val_if_fail (type_reference == NULL || VALA_IS_DATA_TYPE (type_reference), NULL);
+	g_return_val_if_fail (collection == NULL || VALA_IS_EXPRESSION (collection), NULL);
 	g_return_val_if_fail (body == NULL || VALA_IS_BLOCK (body), NULL);
 	g_return_val_if_fail (source_reference == NULL || VALA_IS_SOURCE_REFERENCE (source_reference), NULL);
-	__params = g_new0 (GParameter, 3);
+	__params = g_new0 (GParameter, 2);
 	__params_it = __params;
-	(__params_it->name = "type-reference", g_value_init (&__params_it->value, VALA_TYPE_TYPE_REFERENCE), g_value_set_object (&__params_it->value, type_reference), __params_it++);
 	(__params_it->name = "variable-name", g_value_init (&__params_it->value, G_TYPE_STRING), g_value_set_string (&__params_it->value, variable_name), __params_it++);
 	(__params_it->name = "collection", g_value_init (&__params_it->value, VALA_TYPE_EXPRESSION), g_value_set_object (&__params_it->value, collection), __params_it++);
 	self = g_object_newv (VALA_TYPE_FOREACH_STATEMENT, __params_it - __params, __params);
+	vala_foreach_statement_set_type_reference (self, type_reference);
 	vala_foreach_statement_set_body (self, body);
 	vala_code_node_set_source_reference (VALA_CODE_NODE (self), source_reference);
 	while (__params_it > __params) {
@@ -92,7 +92,7 @@ ValaForeachStatement* vala_foreach_statement_new (ValaTypeReference* type_refere
 static void vala_foreach_statement_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor) {
 	ValaForeachStatement * self;
 	self = VALA_FOREACH_STATEMENT (base);
-	g_return_if_fail (VALA_IS_CODE_VISITOR (visitor));
+	g_return_if_fail (visitor == NULL || VALA_IS_CODE_VISITOR (visitor));
 	vala_code_visitor_visit_foreach_statement (visitor, self);
 }
 
@@ -100,7 +100,7 @@ static void vala_foreach_statement_real_accept (ValaCodeNode* base, ValaCodeVisi
 static void vala_foreach_statement_real_accept_children (ValaCodeNode* base, ValaCodeVisitor* visitor) {
 	ValaForeachStatement * self;
 	self = VALA_FOREACH_STATEMENT (base);
-	g_return_if_fail (VALA_IS_CODE_VISITOR (visitor));
+	g_return_if_fail (visitor == NULL || VALA_IS_CODE_VISITOR (visitor));
 	vala_code_node_accept (VALA_CODE_NODE (vala_foreach_statement_get_type_reference (self)), visitor);
 	vala_code_node_accept (VALA_CODE_NODE (vala_foreach_statement_get_collection (self)), visitor);
 	vala_code_visitor_visit_end_full_expression (visitor, vala_foreach_statement_get_collection (self));
@@ -108,28 +108,42 @@ static void vala_foreach_statement_real_accept_children (ValaCodeNode* base, Val
 }
 
 
-static void vala_foreach_statement_real_replace (ValaCodeNode* base, ValaCodeNode* old_node, ValaCodeNode* new_node) {
+static void vala_foreach_statement_real_replace_expression (ValaCodeNode* base, ValaExpression* old_node, ValaExpression* new_node) {
 	ValaForeachStatement * self;
 	self = VALA_FOREACH_STATEMENT (base);
-	g_return_if_fail (VALA_IS_CODE_NODE (old_node));
-	g_return_if_fail (VALA_IS_CODE_NODE (new_node));
-	if (VALA_CODE_NODE (vala_foreach_statement_get_collection (self)) == old_node) {
-		vala_foreach_statement_set_collection (self, VALA_EXPRESSION (new_node));
+	g_return_if_fail (old_node == NULL || VALA_IS_EXPRESSION (old_node));
+	g_return_if_fail (new_node == NULL || VALA_IS_EXPRESSION (new_node));
+	if (vala_foreach_statement_get_collection (self) == old_node) {
+		vala_foreach_statement_set_collection (self, new_node);
 	}
 }
 
 
-ValaTypeReference* vala_foreach_statement_get_type_reference (ValaForeachStatement* self) {
-	g_return_val_if_fail (VALA_IS_FOREACH_STATEMENT (self), NULL);
-	return self->priv->_type_reference;
+static void vala_foreach_statement_real_replace_type (ValaCodeNode* base, ValaDataType* old_type, ValaDataType* new_type) {
+	ValaForeachStatement * self;
+	self = VALA_FOREACH_STATEMENT (base);
+	g_return_if_fail (old_type == NULL || VALA_IS_DATA_TYPE (old_type));
+	g_return_if_fail (new_type == NULL || VALA_IS_DATA_TYPE (new_type));
+	if (vala_foreach_statement_get_type_reference (self) == old_type) {
+		vala_foreach_statement_set_type_reference (self, new_type);
+	}
 }
 
 
-void vala_foreach_statement_set_type_reference (ValaForeachStatement* self, ValaTypeReference* value) {
-	ValaTypeReference* _tmp1;
+ValaDataType* vala_foreach_statement_get_type_reference (ValaForeachStatement* self) {
+	g_return_val_if_fail (VALA_IS_FOREACH_STATEMENT (self), NULL);
+	return self->priv->_data_type;
+}
+
+
+void vala_foreach_statement_set_type_reference (ValaForeachStatement* self, ValaDataType* value) {
+	ValaDataType* _tmp2;
+	ValaDataType* _tmp1;
 	g_return_if_fail (VALA_IS_FOREACH_STATEMENT (self));
+	_tmp2 = NULL;
 	_tmp1 = NULL;
-	self->priv->_type_reference = (_tmp1 = g_object_ref (value), (self->priv->_type_reference == NULL ? NULL : (self->priv->_type_reference = (g_object_unref (self->priv->_type_reference), NULL))), _tmp1);
+	self->priv->_data_type = (_tmp2 = (_tmp1 = value, (_tmp1 == NULL ? NULL : g_object_ref (_tmp1))), (self->priv->_data_type == NULL ? NULL : (self->priv->_data_type = (g_object_unref (self->priv->_data_type), NULL))), _tmp2);
+	vala_code_node_set_parent_node (VALA_CODE_NODE (self->priv->_data_type), VALA_CODE_NODE (self));
 }
 
 
@@ -140,10 +154,12 @@ char* vala_foreach_statement_get_variable_name (ValaForeachStatement* self) {
 
 
 void vala_foreach_statement_set_variable_name (ValaForeachStatement* self, const char* value) {
-	char* _tmp1;
+	char* _tmp2;
+	const char* _tmp1;
 	g_return_if_fail (VALA_IS_FOREACH_STATEMENT (self));
+	_tmp2 = NULL;
 	_tmp1 = NULL;
-	self->priv->_variable_name = (_tmp1 = g_strdup (value), (self->priv->_variable_name = (g_free (self->priv->_variable_name), NULL)), _tmp1);
+	self->priv->_variable_name = (_tmp2 = (_tmp1 = value, (_tmp1 == NULL ? NULL : g_strdup (_tmp1))), (self->priv->_variable_name = (g_free (self->priv->_variable_name), NULL)), _tmp2);
 }
 
 
@@ -154,10 +170,12 @@ ValaExpression* vala_foreach_statement_get_collection (ValaForeachStatement* sel
 
 
 void vala_foreach_statement_set_collection (ValaForeachStatement* self, ValaExpression* value) {
+	ValaExpression* _tmp2;
 	ValaExpression* _tmp1;
 	g_return_if_fail (VALA_IS_FOREACH_STATEMENT (self));
+	_tmp2 = NULL;
 	_tmp1 = NULL;
-	self->priv->_collection = (_tmp1 = g_object_ref (value), (self->priv->_collection == NULL ? NULL : (self->priv->_collection = (g_object_unref (self->priv->_collection), NULL))), _tmp1);
+	self->priv->_collection = (_tmp2 = (_tmp1 = value, (_tmp1 == NULL ? NULL : g_object_ref (_tmp1))), (self->priv->_collection == NULL ? NULL : (self->priv->_collection = (g_object_unref (self->priv->_collection), NULL))), _tmp2);
 	vala_code_node_set_parent_node (VALA_CODE_NODE (self->priv->_collection), VALA_CODE_NODE (self));
 }
 
@@ -299,8 +317,9 @@ static void vala_foreach_statement_class_init (ValaForeachStatementClass * klass
 	G_OBJECT_CLASS (klass)->dispose = vala_foreach_statement_dispose;
 	VALA_CODE_NODE_CLASS (klass)->accept = vala_foreach_statement_real_accept;
 	VALA_CODE_NODE_CLASS (klass)->accept_children = vala_foreach_statement_real_accept_children;
-	VALA_CODE_NODE_CLASS (klass)->replace = vala_foreach_statement_real_replace;
-	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_FOREACH_STATEMENT_TYPE_REFERENCE, g_param_spec_object ("type-reference", "type-reference", "type-reference", VALA_TYPE_TYPE_REFERENCE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT));
+	VALA_CODE_NODE_CLASS (klass)->replace_expression = vala_foreach_statement_real_replace_expression;
+	VALA_CODE_NODE_CLASS (klass)->replace_type = vala_foreach_statement_real_replace_type;
+	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_FOREACH_STATEMENT_TYPE_REFERENCE, g_param_spec_object ("type-reference", "type-reference", "type-reference", VALA_TYPE_DATA_TYPE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
 	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_FOREACH_STATEMENT_VARIABLE_NAME, g_param_spec_string ("variable-name", "variable-name", "variable-name", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT));
 	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_FOREACH_STATEMENT_COLLECTION, g_param_spec_object ("collection", "collection", "collection", VALA_TYPE_EXPRESSION, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT));
 	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_FOREACH_STATEMENT_BODY, g_param_spec_object ("body", "body", "body", VALA_TYPE_BLOCK, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
@@ -318,13 +337,13 @@ static void vala_foreach_statement_init (ValaForeachStatement * self) {
 static void vala_foreach_statement_dispose (GObject * obj) {
 	ValaForeachStatement * self;
 	self = VALA_FOREACH_STATEMENT (obj);
-	(self->priv->_type_reference == NULL ? NULL : (self->priv->_type_reference = (g_object_unref (self->priv->_type_reference), NULL)));
 	(self->priv->_variable_name = (g_free (self->priv->_variable_name), NULL));
 	(self->priv->_variable_declarator == NULL ? NULL : (self->priv->_variable_declarator = (g_object_unref (self->priv->_variable_declarator), NULL)));
 	(self->priv->_collection_variable_declarator == NULL ? NULL : (self->priv->_collection_variable_declarator = (g_object_unref (self->priv->_collection_variable_declarator), NULL)));
 	(self->priv->_iterator_variable_declarator == NULL ? NULL : (self->priv->_iterator_variable_declarator = (g_object_unref (self->priv->_iterator_variable_declarator), NULL)));
 	(self->priv->_collection == NULL ? NULL : (self->priv->_collection = (g_object_unref (self->priv->_collection), NULL)));
 	(self->priv->_body == NULL ? NULL : (self->priv->_body = (g_object_unref (self->priv->_body), NULL)));
+	(self->priv->_data_type == NULL ? NULL : (self->priv->_data_type = (g_object_unref (self->priv->_data_type), NULL)));
 	G_OBJECT_CLASS (vala_foreach_statement_parent_class)->dispose (obj);
 }
 
