@@ -1,11 +1,11 @@
 /* valainterfacewriter.vala
  *
- * Copyright (C) 2006-2007  Jürg Billeter, Raffaele Sandrini
+ * Copyright (C) 2006-2008  Jürg Billeter, Raffaele Sandrini
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * version 2.1 of the License, or (at your option) any later version.
 
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -21,8 +21,8 @@
  *	Raffaele Sandrini <rasa@gmx.ch>
  */
 
-#ifndef __VALAINTERFACEWRITER_H__
-#define __VALAINTERFACEWRITER_H__
+#ifndef __VALA_VALAINTERFACEWRITER_H__
+#define __VALA_VALAINTERFACEWRITER_H__
 
 #include <glib.h>
 #include <glib-object.h>
@@ -38,7 +38,7 @@
 #include <vala/valaenumvalue.h>
 #include <vala/valaconstant.h>
 #include <vala/valafield.h>
-#include <vala/valacallback.h>
+#include <vala/valadelegate.h>
 #include <vala/valamethod.h>
 #include <vala/valacreationmethod.h>
 #include <vala/valaproperty.h>
@@ -62,14 +62,15 @@ typedef struct _ValaInterfaceWriterPrivate ValaInterfaceWriterPrivate;
  * Code visitor generating Vala API file for the public interface.
  */
 struct _ValaInterfaceWriter {
-	ValaCodeVisitor parent;
+	ValaCodeVisitor parent_instance;
 	ValaInterfaceWriterPrivate * priv;
 };
 struct _ValaInterfaceWriterClass {
-	ValaCodeVisitorClass parent;
+	ValaCodeVisitorClass parent_class;
 };
 
 void vala_interface_writer_write_file (ValaInterfaceWriter* self, ValaCodeContext* context, const char* filename);
+ValaInterfaceWriter* vala_interface_writer_new (void);
 GType vala_interface_writer_get_type (void);
 
 G_END_DECLS
