@@ -44,6 +44,7 @@ enum  {
 };
 static void vala_lambda_expression_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor);
 static void vala_lambda_expression_real_accept_children (ValaCodeNode* base, ValaCodeVisitor* visitor);
+static gboolean vala_lambda_expression_real_is_pure (ValaExpression* base);
 static gpointer vala_lambda_expression_parent_class = NULL;
 static void vala_lambda_expression_dispose (GObject * obj);
 
@@ -57,7 +58,7 @@ static void vala_lambda_expression_dispose (GObject * obj);
  */
 ValaLambdaExpression* vala_lambda_expression_new (ValaExpression* expression_body, ValaSourceReference* source_reference) {
 	ValaLambdaExpression * self;
-	g_return_val_if_fail (VALA_IS_EXPRESSION (expression_body), NULL);
+	g_return_val_if_fail (expression_body == NULL || VALA_IS_EXPRESSION (expression_body), NULL);
 	g_return_val_if_fail (source_reference == NULL || VALA_IS_SOURCE_REFERENCE (source_reference), NULL);
 	self = g_object_newv (VALA_TYPE_LAMBDA_EXPRESSION, 0, NULL);
 	vala_lambda_expression_set_expression_body (self, expression_body);
@@ -75,7 +76,7 @@ ValaLambdaExpression* vala_lambda_expression_new (ValaExpression* expression_bod
  */
 ValaLambdaExpression* vala_lambda_expression_new_with_statement_body (ValaBlock* statement_body, ValaSourceReference* source_reference) {
 	ValaLambdaExpression * self;
-	g_return_val_if_fail (VALA_IS_BLOCK (statement_body), NULL);
+	g_return_val_if_fail (statement_body == NULL || VALA_IS_BLOCK (statement_body), NULL);
 	g_return_val_if_fail (source_reference == NULL || VALA_IS_SOURCE_REFERENCE (source_reference), NULL);
 	self = g_object_newv (VALA_TYPE_LAMBDA_EXPRESSION, 0, NULL);
 	vala_lambda_expression_set_statement_body (self, statement_body);
@@ -91,7 +92,6 @@ ValaLambdaExpression* vala_lambda_expression_new_with_statement_body (ValaBlock*
  */
 void vala_lambda_expression_add_parameter (ValaLambdaExpression* self, const char* param) {
 	g_return_if_fail (VALA_IS_LAMBDA_EXPRESSION (self));
-	g_return_if_fail (param != NULL);
 	gee_collection_add (GEE_COLLECTION (self->priv->parameters), param);
 }
 
@@ -110,7 +110,7 @@ GeeCollection* vala_lambda_expression_get_parameters (ValaLambdaExpression* self
 static void vala_lambda_expression_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor) {
 	ValaLambdaExpression * self;
 	self = VALA_LAMBDA_EXPRESSION (base);
-	g_return_if_fail (VALA_IS_CODE_VISITOR (visitor));
+	g_return_if_fail (visitor == NULL || VALA_IS_CODE_VISITOR (visitor));
 	vala_code_visitor_visit_lambda_expression (visitor, self);
 }
 
@@ -118,7 +118,7 @@ static void vala_lambda_expression_real_accept (ValaCodeNode* base, ValaCodeVisi
 static void vala_lambda_expression_real_accept_children (ValaCodeNode* base, ValaCodeVisitor* visitor) {
 	ValaLambdaExpression * self;
 	self = VALA_LAMBDA_EXPRESSION (base);
-	g_return_if_fail (VALA_IS_CODE_VISITOR (visitor));
+	g_return_if_fail (visitor == NULL || VALA_IS_CODE_VISITOR (visitor));
 	if (vala_lambda_expression_get_method (self) == NULL) {
 		if (vala_lambda_expression_get_expression_body (self) != NULL) {
 			vala_code_node_accept (VALA_CODE_NODE (vala_lambda_expression_get_expression_body (self)), visitor);
@@ -134,6 +134,13 @@ static void vala_lambda_expression_real_accept_children (ValaCodeNode* base, Val
 }
 
 
+static gboolean vala_lambda_expression_real_is_pure (ValaExpression* base) {
+	ValaLambdaExpression * self;
+	self = VALA_LAMBDA_EXPRESSION (base);
+	return FALSE;
+}
+
+
 ValaExpression* vala_lambda_expression_get_expression_body (ValaLambdaExpression* self) {
 	g_return_val_if_fail (VALA_IS_LAMBDA_EXPRESSION (self), NULL);
 	return self->priv->_expression_body;
@@ -230,6 +237,7 @@ static void vala_lambda_expression_class_init (ValaLambdaExpressionClass * klass
 	G_OBJECT_CLASS (klass)->dispose = vala_lambda_expression_dispose;
 	VALA_CODE_NODE_CLASS (klass)->accept = vala_lambda_expression_real_accept;
 	VALA_CODE_NODE_CLASS (klass)->accept_children = vala_lambda_expression_real_accept_children;
+	VALA_EXPRESSION_CLASS (klass)->is_pure = vala_lambda_expression_real_is_pure;
 	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_LAMBDA_EXPRESSION_EXPRESSION_BODY, g_param_spec_object ("expression-body", "expression-body", "expression-body", VALA_TYPE_EXPRESSION, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
 	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_LAMBDA_EXPRESSION_STATEMENT_BODY, g_param_spec_object ("statement-body", "statement-body", "statement-body", VALA_TYPE_BLOCK, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
 	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_LAMBDA_EXPRESSION_METHOD, g_param_spec_object ("method", "method", "method", VALA_TYPE_METHOD, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
