@@ -29,7 +29,9 @@
  * Indicates a specific lockable object beeing actually locked somewhere.
  */
 gboolean vala_lockable_get_lock_used (ValaLockable* self) {
-	return VALA_LOCKABLE_GET_INTERFACE (self)->get_lock_used (self);
+	gboolean result;
+	result = VALA_LOCKABLE_GET_INTERFACE (self)->get_lock_used (self);
+	return result;
 }
 
 
