@@ -1,6 +1,6 @@
 /* valanamespace.vala
  *
- * Copyright (C) 2006-2007  Jürg Billeter
+ * Copyright (C) 2006-2008  Jürg Billeter
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -48,11 +48,11 @@ typedef struct _ValaNamespacePrivate ValaNamespacePrivate;
  * Represents a namespace declaration in the source code.
  */
 struct _ValaNamespace {
-	ValaSymbol parent;
+	ValaSymbol parent_instance;
 	ValaNamespacePrivate * priv;
 };
 struct _ValaNamespaceClass {
-	ValaSymbolClass parent;
+	ValaSymbolClass parent_class;
 };
 
 ValaNamespace* vala_namespace_new (const char* name, ValaSourceReference* source_reference);
@@ -62,7 +62,7 @@ void vala_namespace_add_class (ValaNamespace* self, ValaClass* cl);
 void vala_namespace_add_interface (ValaNamespace* self, ValaInterface* iface);
 void vala_namespace_add_struct (ValaNamespace* self, ValaStruct* st);
 void vala_namespace_add_enum (ValaNamespace* self, ValaEnum* en);
-void vala_namespace_add_callback (ValaNamespace* self, ValaCallback* cb);
+void vala_namespace_add_delegate (ValaNamespace* self, ValaDelegate* d);
 GeeCollection* vala_namespace_get_structs (ValaNamespace* self);
 GeeCollection* vala_namespace_get_classes (ValaNamespace* self);
 GeeCollection* vala_namespace_get_interfaces (ValaNamespace* self);
