@@ -48,20 +48,20 @@ typedef struct _ValaPropertyPrivate ValaPropertyPrivate;
  * Represents a property declaration in the source code.
  */
 struct _ValaProperty {
-	ValaMember parent;
+	ValaMember parent_instance;
 	ValaPropertyPrivate * priv;
 };
 struct _ValaPropertyClass {
-	ValaMemberClass parent;
+	ValaMemberClass parent_class;
 };
 
-ValaProperty* vala_property_new (const char* _name, ValaTypeReference* type, ValaPropertyAccessor* _get_accessor, ValaPropertyAccessor* _set_accessor, ValaSourceReference* source);
+ValaProperty* vala_property_new (const char* _name, ValaDataType* type, ValaPropertyAccessor* _get_accessor, ValaPropertyAccessor* _set_accessor, ValaSourceReference* source);
 char* vala_property_get_upper_case_cname (ValaProperty* self);
 ValaCCodeConstant* vala_property_get_canonical_cconstant (ValaProperty* self);
 void vala_property_process_attributes (ValaProperty* self);
 gboolean vala_property_equals (ValaProperty* self, ValaProperty* prop2);
-ValaTypeReference* vala_property_get_type_reference (ValaProperty* self);
-void vala_property_set_type_reference (ValaProperty* self, ValaTypeReference* value);
+ValaDataType* vala_property_get_type_reference (ValaProperty* self);
+void vala_property_set_type_reference (ValaProperty* self, ValaDataType* value);
 ValaPropertyAccessor* vala_property_get_get_accessor (ValaProperty* self);
 void vala_property_set_get_accessor (ValaProperty* self, ValaPropertyAccessor* value);
 ValaPropertyAccessor* vala_property_get_set_accessor (ValaProperty* self);
