@@ -5,7 +5,7 @@
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * version 2.1 of the License, or (at your option) any later version.
 
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -20,8 +20,8 @@
  * 	Jürg Billeter <j@bitron.ch>
  */
 
-#ifndef __VALAPROPERTYACCESSOR_H__
-#define __VALAPROPERTYACCESSOR_H__
+#ifndef __VALA_VALAPROPERTYACCESSOR_H__
+#define __VALA_VALAPROPERTYACCESSOR_H__
 
 #include <glib.h>
 #include <glib-object.h>
@@ -43,11 +43,11 @@ typedef struct _ValaPropertyAccessorPrivate ValaPropertyAccessorPrivate;
  * Represents a get or set accessor of a property in the source code.
  */
 struct _ValaPropertyAccessor {
-	ValaCodeNode parent;
+	ValaCodeNode parent_instance;
 	ValaPropertyAccessorPrivate * priv;
 };
 struct _ValaPropertyAccessorClass {
-	ValaCodeNodeClass parent;
+	ValaCodeNodeClass parent_class;
 };
 
 ValaPropertyAccessor* vala_property_accessor_new (gboolean readable, gboolean writable, gboolean construction, ValaBlock* body, ValaSourceReference* source_reference);
