@@ -40,7 +40,7 @@ G_BEGIN_DECLS
  * Interface for all statement types.
  */
 struct _ValaStatementIface {
-	GTypeInterface parent;
+	GTypeInterface parent_iface;
 };
 
 GType vala_statement_get_type (void);
