@@ -1,6 +1,6 @@
 /* valastruct.vala
  *
- * Copyright (C) 2006-2007  Jürg Billeter
+ * Copyright (C) 2006-2008  Jürg Billeter
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -28,11 +28,15 @@
 #include <vala/valaconstant.h>
 #include <vala/valafield.h>
 #include <vala/valamethod.h>
-#include <vala/valatypereference.h>
+#include <vala/valadatatype.h>
 #include <vala/valasourcereference.h>
 #include <vala/valasymbol.h>
 #include <vala/valascope.h>
 #include <vala/valaformalparameter.h>
+#include <vala/valavaluetype.h>
+#include <vala/valavoidtype.h>
+#include <vala/valavariabledeclarator.h>
+#include <vala/valaexpression.h>
 #include <vala/valacreationmethod.h>
 #include <vala/valacodevisitor.h>
 #include <vala/valaattribute.h>
@@ -68,26 +72,27 @@ enum  {
 };
 static void vala_struct_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor);
 static void vala_struct_real_accept_children (ValaCodeNode* base, ValaCodeVisitor* visitor);
-static char* vala_struct_real_get_cname (ValaDataType* base, gboolean const_type);
+static char* vala_struct_real_get_cname (ValaTypesymbol* base, gboolean const_type);
 static void vala_struct_set_cname (ValaStruct* self, const char* cname);
 static void vala_struct_set_const_cname (ValaStruct* self, const char* cname);
 static char* vala_struct_real_get_lower_case_cprefix (ValaSymbol* base);
 static char* vala_struct_get_lower_case_csuffix (ValaStruct* self);
 static char* vala_struct_real_get_lower_case_cname (ValaSymbol* base, const char* infix);
-static char* vala_struct_real_get_upper_case_cname (ValaDataType* base, const char* infix);
+static char* vala_struct_real_get_upper_case_cname (ValaTypesymbol* base, const char* infix);
 static void vala_struct_process_ccode_attribute (ValaStruct* self, ValaAttribute* a);
 static void vala_struct_process_integer_type_attribute (ValaStruct* self, ValaAttribute* a);
 static void vala_struct_process_floating_type_attribute (ValaStruct* self, ValaAttribute* a);
-static char* vala_struct_real_get_type_id (ValaDataType* base);
-static char* vala_struct_real_get_marshaller_type_name (ValaDataType* base);
+static char* vala_struct_real_get_type_id (ValaTypesymbol* base);
+static char* vala_struct_real_get_marshaller_type_name (ValaTypesymbol* base);
 static void vala_struct_set_marshaller_type_name (ValaStruct* self, const char* name);
-static char* vala_struct_real_get_get_value_function (ValaDataType* base);
-static char* vala_struct_real_get_set_value_function (ValaDataType* base);
+static char* vala_struct_real_get_get_value_function (ValaTypesymbol* base);
+static char* vala_struct_real_get_set_value_function (ValaTypesymbol* base);
 static void vala_struct_set_get_value_function (ValaStruct* self, const char* function);
 static void vala_struct_set_set_value_function (ValaStruct* self, const char* function);
-static char* vala_struct_real_get_default_value (ValaDataType* base);
+static char* vala_struct_real_get_default_value (ValaTypesymbol* base);
 static void vala_struct_set_default_value (ValaStruct* self, const char* value);
-static gint vala_struct_real_get_type_parameter_index (ValaDataType* base, const char* name);
+static gint vala_struct_real_get_type_parameter_index (ValaTypesymbol* base, const char* name);
+static void vala_struct_real_replace_type (ValaCodeNode* base, ValaDataType* old_type, ValaDataType* new_type);
 static gpointer vala_struct_parent_class = NULL;
 static void vala_struct_dispose (GObject * obj);
 static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func);
@@ -102,7 +107,6 @@ static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify
  */
 ValaStruct* vala_struct_new (const char* name, ValaSourceReference* source_reference) {
 	ValaStruct * self;
-	g_return_val_if_fail (name != NULL, NULL);
 	g_return_val_if_fail (source_reference == NULL || VALA_IS_SOURCE_REFERENCE (source_reference), NULL);
 	self = g_object_newv (VALA_TYPE_STRUCT, 0, NULL);
 	vala_symbol_set_name (VALA_SYMBOL (self), name);
@@ -118,9 +122,9 @@ ValaStruct* vala_struct_new (const char* name, ValaSourceReference* source_refer
  */
 void vala_struct_add_type_parameter (ValaStruct* self, ValaTypeParameter* p) {
 	g_return_if_fail (VALA_IS_STRUCT (self));
-	g_return_if_fail (VALA_IS_TYPE_PARAMETER (p));
+	g_return_if_fail (p == NULL || VALA_IS_TYPEPARAMETER (p));
 	gee_collection_add (GEE_COLLECTION (self->priv->type_parameters), p);
-	p->type = VALA_DATA_TYPE (self);
+	p->type = VALA_TYPESYMBOL (self);
 	vala_scope_add (vala_symbol_get_scope (VALA_SYMBOL (self)), vala_symbol_get_name (VALA_SYMBOL (p)), VALA_SYMBOL (p));
 }
 
@@ -132,7 +136,7 @@ void vala_struct_add_type_parameter (ValaStruct* self, ValaTypeParameter* p) {
  */
 void vala_struct_add_constant (ValaStruct* self, ValaConstant* c) {
 	g_return_if_fail (VALA_IS_STRUCT (self));
-	g_return_if_fail (VALA_IS_CONSTANT (c));
+	g_return_if_fail (c == NULL || VALA_IS_CONSTANT (c));
 	gee_collection_add (GEE_COLLECTION (self->priv->constants), c);
 	vala_scope_add (vala_symbol_get_scope (VALA_SYMBOL (self)), vala_symbol_get_name (VALA_SYMBOL (c)), VALA_SYMBOL (c));
 }
@@ -145,7 +149,7 @@ void vala_struct_add_constant (ValaStruct* self, ValaConstant* c) {
  */
 void vala_struct_add_field (ValaStruct* self, ValaField* f) {
 	g_return_if_fail (VALA_IS_STRUCT (self));
-	g_return_if_fail (VALA_IS_FIELD (f));
+	g_return_if_fail (f == NULL || VALA_IS_FIELD (f));
 	gee_collection_add (GEE_COLLECTION (self->priv->fields), f);
 	vala_scope_add (vala_symbol_get_scope (VALA_SYMBOL (self)), vala_symbol_get_name (VALA_SYMBOL (f)), VALA_SYMBOL (f));
 }
@@ -169,21 +173,39 @@ GeeCollection* vala_struct_get_fields (ValaStruct* self) {
  */
 void vala_struct_add_method (ValaStruct* self, ValaMethod* m) {
 	g_return_if_fail (VALA_IS_STRUCT (self));
-	g_return_if_fail (VALA_IS_METHOD (m));
+	g_return_if_fail (m == NULL || VALA_IS_METHOD (m));
 	g_return_if_fail (m != NULL);
 	if (vala_method_get_instance (m)) {
 		ValaFormalParameter* _tmp1;
-		ValaTypeReference* _tmp0;
+		ValaValueType* _tmp0;
 		_tmp1 = NULL;
 		_tmp0 = NULL;
-		vala_method_set_this_parameter (m, (_tmp1 = vala_formal_parameter_new ("this", (_tmp0 = vala_type_reference_new ()), NULL)));
+		vala_method_set_this_parameter (m, (_tmp1 = vala_formal_parameter_new ("this", VALA_DATA_TYPE ((_tmp0 = vala_value_type_new (VALA_TYPESYMBOL (self)))), NULL)));
 		(_tmp1 == NULL ? NULL : (_tmp1 = (g_object_unref (_tmp1), NULL)));
 		(_tmp0 == NULL ? NULL : (_tmp0 = (g_object_unref (_tmp0), NULL)));
-		vala_type_reference_set_data_type (vala_formal_parameter_get_type_reference (vala_method_get_this_parameter (m)), VALA_DATA_TYPE (self));
 		vala_scope_add (vala_symbol_get_scope (VALA_SYMBOL (m)), vala_symbol_get_name (VALA_SYMBOL (vala_method_get_this_parameter (m))), VALA_SYMBOL (vala_method_get_this_parameter (m)));
 	}
-	if (VALA_IS_CREATION_METHOD (m) && vala_symbol_get_name (VALA_SYMBOL (m)) == NULL) {
-		vala_struct_set_default_construction_method (self, m);
+	if (!(VALA_IS_VOID_TYPE (vala_method_get_return_type (m)))) {
+		ValaVariableDeclarator* _tmp2;
+		ValaDataType* _tmp3;
+		_tmp2 = NULL;
+		vala_method_set_result_var (m, (_tmp2 = vala_variable_declarator_new ("result", NULL, NULL)));
+		(_tmp2 == NULL ? NULL : (_tmp2 = (g_object_unref (_tmp2), NULL)));
+		_tmp3 = NULL;
+		vala_variable_declarator_set_type_reference (vala_method_get_result_var (m), (_tmp3 = vala_data_type_copy (vala_method_get_return_type (m))));
+		(_tmp3 == NULL ? NULL : (_tmp3 = (g_object_unref (_tmp3), NULL)));
+		vala_scope_add (vala_symbol_get_scope (VALA_SYMBOL (m)), vala_symbol_get_name (VALA_SYMBOL (vala_method_get_result_var (m))), VALA_SYMBOL (vala_method_get_result_var (m)));
+	}
+	if (VALA_IS_CREATION_METHOD (m)) {
+		if (vala_symbol_get_name (VALA_SYMBOL (m)) == NULL) {
+			vala_struct_set_default_construction_method (self, m);
+			vala_symbol_set_name (VALA_SYMBOL (m), ".new");
+		} else {
+			char* _tmp4;
+			_tmp4 = NULL;
+			vala_symbol_set_name (VALA_SYMBOL (m), (_tmp4 = g_strconcat (".new.", vala_symbol_get_name (VALA_SYMBOL (m)), NULL)));
+			(_tmp4 = (g_free (_tmp4), NULL));
+		}
 	}
 	gee_collection_add (GEE_COLLECTION (self->priv->methods), m);
 	vala_scope_add (vala_symbol_get_scope (VALA_SYMBOL (self)), vala_symbol_get_name (VALA_SYMBOL (m)), VALA_SYMBOL (m));
@@ -204,7 +226,7 @@ GeeCollection* vala_struct_get_methods (ValaStruct* self) {
 static void vala_struct_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor) {
 	ValaStruct * self;
 	self = VALA_STRUCT (base);
-	g_return_if_fail (VALA_IS_CODE_VISITOR (visitor));
+	g_return_if_fail (visitor == NULL || VALA_IS_CODE_VISITOR (visitor));
 	vala_code_visitor_visit_struct (visitor, self);
 }
 
@@ -212,14 +234,14 @@ static void vala_struct_real_accept (ValaCodeNode* base, ValaCodeVisitor* visito
 static void vala_struct_real_accept_children (ValaCodeNode* base, ValaCodeVisitor* visitor) {
 	ValaStruct * self;
 	self = VALA_STRUCT (base);
-	g_return_if_fail (VALA_IS_CODE_VISITOR (visitor));
+	g_return_if_fail (visitor == NULL || VALA_IS_CODE_VISITOR (visitor));
 	{
 		GeeList* type_collection;
 		GeeIterator* type_it;
 		type_collection = self->priv->base_types;
 		type_it = gee_iterable_iterator (GEE_ITERABLE (type_collection));
 		while (gee_iterator_next (type_it)) {
-			ValaTypeReference* type;
+			ValaDataType* type;
 			type = gee_iterator_get (type_it);
 			{
 				vala_code_node_accept (VALA_CODE_NODE (type), visitor);
@@ -291,7 +313,7 @@ static void vala_struct_real_accept_children (ValaCodeNode* base, ValaCodeVisito
 }
 
 
-static char* vala_struct_real_get_cname (ValaDataType* base, gboolean const_type) {
+static char* vala_struct_real_get_cname (ValaTypesymbol* base, gboolean const_type) {
 	ValaStruct * self;
 	const char* _tmp4;
 	self = VALA_STRUCT (base);
@@ -314,20 +336,22 @@ static char* vala_struct_real_get_cname (ValaDataType* base, gboolean const_type
 
 
 static void vala_struct_set_cname (ValaStruct* self, const char* cname) {
-	char* _tmp0;
+	char* _tmp1;
+	const char* _tmp0;
 	g_return_if_fail (VALA_IS_STRUCT (self));
-	g_return_if_fail (cname != NULL);
+	_tmp1 = NULL;
 	_tmp0 = NULL;
-	self->priv->cname = (_tmp0 = g_strdup (cname), (self->priv->cname = (g_free (self->priv->cname), NULL)), _tmp0);
+	self->priv->cname = (_tmp1 = (_tmp0 = cname, (_tmp0 == NULL ? NULL : g_strdup (_tmp0))), (self->priv->cname = (g_free (self->priv->cname), NULL)), _tmp1);
 }
 
 
 static void vala_struct_set_const_cname (ValaStruct* self, const char* cname) {
-	char* _tmp0;
+	char* _tmp1;
+	const char* _tmp0;
 	g_return_if_fail (VALA_IS_STRUCT (self));
-	g_return_if_fail (cname != NULL);
+	_tmp1 = NULL;
 	_tmp0 = NULL;
-	self->priv->const_cname = (_tmp0 = g_strdup (cname), (self->priv->const_cname = (g_free (self->priv->const_cname), NULL)), _tmp0);
+	self->priv->const_cname = (_tmp1 = (_tmp0 = cname, (_tmp0 == NULL ? NULL : g_strdup (_tmp0))), (self->priv->const_cname = (g_free (self->priv->const_cname), NULL)), _tmp1);
 }
 
 
@@ -377,7 +401,7 @@ static char* vala_struct_real_get_lower_case_cname (ValaSymbol* base, const char
 }
 
 
-static char* vala_struct_real_get_upper_case_cname (ValaDataType* base, const char* infix) {
+static char* vala_struct_real_get_upper_case_cname (ValaTypesymbol* base, const char* infix) {
 	ValaStruct * self;
 	char* _tmp0;
 	char* _tmp1;
@@ -423,7 +447,7 @@ gint vala_struct_get_rank (ValaStruct* self) {
 
 static void vala_struct_process_ccode_attribute (ValaStruct* self, ValaAttribute* a) {
 	g_return_if_fail (VALA_IS_STRUCT (self));
-	g_return_if_fail (VALA_IS_ATTRIBUTE (a));
+	g_return_if_fail (a == NULL || VALA_IS_ATTRIBUTE (a));
 	if (vala_attribute_has_argument (a, "cname")) {
 		char* _tmp0;
 		_tmp0 = NULL;
@@ -456,7 +480,7 @@ static void vala_struct_process_ccode_attribute (ValaStruct* self, ValaAttribute
 				_tmp3 = NULL;
 				filename = (_tmp3 = *filename_it, (_tmp3 == NULL ? NULL : g_strdup (_tmp3)));
 				{
-					vala_data_type_add_cheader_filename (VALA_DATA_TYPE (self), filename);
+					vala_typesymbol_add_cheader_filename (VALA_TYPESYMBOL (self), filename);
 					(filename = (g_free (filename), NULL));
 				}
 			}
@@ -499,7 +523,7 @@ static void vala_struct_process_ccode_attribute (ValaStruct* self, ValaAttribute
 
 static void vala_struct_process_integer_type_attribute (ValaStruct* self, ValaAttribute* a) {
 	g_return_if_fail (VALA_IS_STRUCT (self));
-	g_return_if_fail (VALA_IS_ATTRIBUTE (a));
+	g_return_if_fail (a == NULL || VALA_IS_ATTRIBUTE (a));
 	self->priv->integer_type = TRUE;
 	if (vala_attribute_has_argument (a, "rank")) {
 		self->priv->rank = vala_attribute_get_integer (a, "rank");
@@ -509,7 +533,7 @@ static void vala_struct_process_integer_type_attribute (ValaStruct* self, ValaAt
 
 static void vala_struct_process_floating_type_attribute (ValaStruct* self, ValaAttribute* a) {
 	g_return_if_fail (VALA_IS_STRUCT (self));
-	g_return_if_fail (VALA_IS_ATTRIBUTE (a));
+	g_return_if_fail (a == NULL || VALA_IS_ATTRIBUTE (a));
 	self->priv->floating_type = TRUE;
 	if (vala_attribute_has_argument (a, "rank")) {
 		self->priv->rank = vala_attribute_get_integer (a, "rank");
@@ -554,7 +578,7 @@ void vala_struct_process_attributes (ValaStruct* self) {
 }
 
 
-static char* vala_struct_real_get_type_id (ValaDataType* base) {
+static char* vala_struct_real_get_type_id (ValaTypesymbol* base) {
 	ValaStruct * self;
 	const char* _tmp2;
 	self = VALA_STRUCT (base);
@@ -573,15 +597,16 @@ static char* vala_struct_real_get_type_id (ValaDataType* base) {
 
 
 void vala_struct_set_type_id (ValaStruct* self, const char* name) {
-	char* _tmp0;
+	char* _tmp1;
+	const char* _tmp0;
 	g_return_if_fail (VALA_IS_STRUCT (self));
-	g_return_if_fail (name != NULL);
+	_tmp1 = NULL;
 	_tmp0 = NULL;
-	self->priv->type_id = (_tmp0 = g_strdup (name), (self->priv->type_id = (g_free (self->priv->type_id), NULL)), _tmp0);
+	self->priv->type_id = (_tmp1 = (_tmp0 = name, (_tmp0 == NULL ? NULL : g_strdup (_tmp0))), (self->priv->type_id = (g_free (self->priv->type_id), NULL)), _tmp1);
 }
 
 
-static char* vala_struct_real_get_marshaller_type_name (ValaDataType* base) {
+static char* vala_struct_real_get_marshaller_type_name (ValaTypesymbol* base) {
 	ValaStruct * self;
 	const char* _tmp2;
 	self = VALA_STRUCT (base);
@@ -600,15 +625,16 @@ static char* vala_struct_real_get_marshaller_type_name (ValaDataType* base) {
 
 
 static void vala_struct_set_marshaller_type_name (ValaStruct* self, const char* name) {
-	char* _tmp0;
+	char* _tmp1;
+	const char* _tmp0;
 	g_return_if_fail (VALA_IS_STRUCT (self));
-	g_return_if_fail (name != NULL);
+	_tmp1 = NULL;
 	_tmp0 = NULL;
-	self->priv->marshaller_type_name = (_tmp0 = g_strdup (name), (self->priv->marshaller_type_name = (g_free (self->priv->marshaller_type_name), NULL)), _tmp0);
+	self->priv->marshaller_type_name = (_tmp1 = (_tmp0 = name, (_tmp0 == NULL ? NULL : g_strdup (_tmp0))), (self->priv->marshaller_type_name = (g_free (self->priv->marshaller_type_name), NULL)), _tmp1);
 }
 
 
-static char* vala_struct_real_get_get_value_function (ValaDataType* base) {
+static char* vala_struct_real_get_get_value_function (ValaTypesymbol* base) {
 	ValaStruct * self;
 	self = VALA_STRUCT (base);
 	if (self->priv->get_value_function == NULL) {
@@ -628,7 +654,7 @@ static char* vala_struct_real_get_get_value_function (ValaDataType* base) {
 }
 
 
-static char* vala_struct_real_get_set_value_function (ValaDataType* base) {
+static char* vala_struct_real_get_set_value_function (ValaTypesymbol* base) {
 	ValaStruct * self;
 	self = VALA_STRUCT (base);
 	if (self->priv->set_value_function == NULL) {
@@ -649,24 +675,26 @@ static char* vala_struct_real_get_set_value_function (ValaDataType* base) {
 
 
 static void vala_struct_set_get_value_function (ValaStruct* self, const char* function) {
-	char* _tmp0;
+	char* _tmp1;
+	const char* _tmp0;
 	g_return_if_fail (VALA_IS_STRUCT (self));
-	g_return_if_fail (function != NULL);
+	_tmp1 = NULL;
 	_tmp0 = NULL;
-	self->priv->get_value_function = (_tmp0 = g_strdup (function), (self->priv->get_value_function = (g_free (self->priv->get_value_function), NULL)), _tmp0);
+	self->priv->get_value_function = (_tmp1 = (_tmp0 = function, (_tmp0 == NULL ? NULL : g_strdup (_tmp0))), (self->priv->get_value_function = (g_free (self->priv->get_value_function), NULL)), _tmp1);
 }
 
 
 static void vala_struct_set_set_value_function (ValaStruct* self, const char* function) {
-	char* _tmp0;
+	char* _tmp1;
+	const char* _tmp0;
 	g_return_if_fail (VALA_IS_STRUCT (self));
-	g_return_if_fail (function != NULL);
+	_tmp1 = NULL;
 	_tmp0 = NULL;
-	self->priv->set_value_function = (_tmp0 = g_strdup (function), (self->priv->set_value_function = (g_free (self->priv->set_value_function), NULL)), _tmp0);
+	self->priv->set_value_function = (_tmp1 = (_tmp0 = function, (_tmp0 == NULL ? NULL : g_strdup (_tmp0))), (self->priv->set_value_function = (g_free (self->priv->set_value_function), NULL)), _tmp1);
 }
 
 
-static char* vala_struct_real_get_default_value (ValaDataType* base) {
+static char* vala_struct_real_get_default_value (ValaTypesymbol* base) {
 	ValaStruct * self;
 	const char* _tmp0;
 	self = VALA_STRUCT (base);
@@ -676,11 +704,12 @@ static char* vala_struct_real_get_default_value (ValaDataType* base) {
 
 
 static void vala_struct_set_default_value (ValaStruct* self, const char* value) {
-	char* _tmp0;
+	char* _tmp1;
+	const char* _tmp0;
 	g_return_if_fail (VALA_IS_STRUCT (self));
-	g_return_if_fail (value != NULL);
+	_tmp1 = NULL;
 	_tmp0 = NULL;
-	self->priv->default_value = (_tmp0 = g_strdup (value), (self->priv->default_value = (g_free (self->priv->default_value), NULL)), _tmp0);
+	self->priv->default_value = (_tmp1 = (_tmp0 = value, (_tmp0 == NULL ? NULL : g_strdup (_tmp0))), (self->priv->default_value = (g_free (self->priv->default_value), NULL)), _tmp1);
 }
 
 
@@ -689,10 +718,11 @@ static void vala_struct_set_default_value (ValaStruct* self, const char* value)
  *
  * @param type a class or interface reference
  */
-void vala_struct_add_base_type (ValaStruct* self, ValaTypeReference* type) {
+void vala_struct_add_base_type (ValaStruct* self, ValaDataType* type) {
 	g_return_if_fail (VALA_IS_STRUCT (self));
-	g_return_if_fail (VALA_IS_TYPE_REFERENCE (type));
+	g_return_if_fail (type == NULL || VALA_IS_DATA_TYPE (type));
 	gee_collection_add (GEE_COLLECTION (self->priv->base_types), type);
+	vala_code_node_set_parent_node (VALA_CODE_NODE (type), VALA_CODE_NODE (self));
 }
 
 
@@ -703,15 +733,14 @@ void vala_struct_add_base_type (ValaStruct* self, ValaTypeReference* type) {
  */
 GeeCollection* vala_struct_get_base_types (ValaStruct* self) {
 	g_return_val_if_fail (VALA_IS_STRUCT (self), NULL);
-	return GEE_COLLECTION (gee_read_only_collection_new (VALA_TYPE_TYPE_REFERENCE, ((GBoxedCopyFunc) g_object_ref), g_object_unref, GEE_COLLECTION (self->priv->base_types)));
+	return GEE_COLLECTION (gee_read_only_collection_new (VALA_TYPE_DATA_TYPE, ((GBoxedCopyFunc) g_object_ref), g_object_unref, GEE_COLLECTION (self->priv->base_types)));
 }
 
 
-static gint vala_struct_real_get_type_parameter_index (ValaDataType* base, const char* name) {
+static gint vala_struct_real_get_type_parameter_index (ValaTypesymbol* base, const char* name) {
 	ValaStruct * self;
 	gint i;
 	self = VALA_STRUCT (base);
-	g_return_val_if_fail (name != NULL, 0);
 	i = 0;
 	{
 		GeeList* p_collection;
@@ -748,15 +777,15 @@ gboolean vala_struct_is_simple_type (ValaStruct* self) {
 		type_collection = self->priv->base_types;
 		type_it = gee_iterable_iterator (GEE_ITERABLE (type_collection));
 		while (gee_iterator_next (type_it)) {
-			ValaTypeReference* type;
+			ValaDataType* type;
 			type = gee_iterator_get (type_it);
 			{
 				ValaStruct* _tmp1;
-				ValaDataType* _tmp0;
+				ValaTypesymbol* _tmp0;
 				ValaStruct* st;
 				_tmp1 = NULL;
 				_tmp0 = NULL;
-				st = (_tmp1 = (_tmp0 = vala_type_reference_get_data_type (type), (VALA_IS_STRUCT (_tmp0) ? ((ValaStruct*) _tmp0) : NULL)), (_tmp1 == NULL ? NULL : g_object_ref (_tmp1)));
+				st = (_tmp1 = (_tmp0 = vala_data_type_get_data_type (type), (VALA_IS_STRUCT (_tmp0) ? ((ValaStruct*) _tmp0) : NULL)), (_tmp1 == NULL ? NULL : g_object_ref (_tmp1)));
 				if (st != NULL && vala_struct_is_simple_type (st)) {
 					gboolean _tmp2;
 					return (_tmp2 = TRUE, (type == NULL ? NULL : (type = (g_object_unref (type), NULL))), (st == NULL ? NULL : (st = (g_object_unref (st), NULL))), (type_it == NULL ? NULL : (type_it = (g_object_unref (type_it), NULL))), _tmp2);
@@ -771,6 +800,37 @@ gboolean vala_struct_is_simple_type (ValaStruct* self) {
 }
 
 
+/**
+ * Marks this struct as simple type, i.e. instances will be passed by
+ * value.
+ */
+void vala_struct_set_simple_type (ValaStruct* self, gboolean simple_type) {
+	g_return_if_fail (VALA_IS_STRUCT (self));
+	self->priv->simple_type = simple_type;
+}
+
+
+static void vala_struct_real_replace_type (ValaCodeNode* base, ValaDataType* old_type, ValaDataType* new_type) {
+	ValaStruct * self;
+	self = VALA_STRUCT (base);
+	g_return_if_fail (old_type == NULL || VALA_IS_DATA_TYPE (old_type));
+	g_return_if_fail (new_type == NULL || VALA_IS_DATA_TYPE (new_type));
+	{
+		gint i;
+		i = 0;
+		for (i = 0; i < gee_collection_get_size (GEE_COLLECTION (self->priv->base_types)); i++) {
+			ValaDataType* _tmp0;
+			gboolean _tmp1;
+			_tmp0 = NULL;
+			if ((_tmp1 = (_tmp0 = gee_list_get (((GeeList*) self->priv->base_types), i)) == old_type, (_tmp0 == NULL ? NULL : (_tmp0 = (g_object_unref (_tmp0), NULL))), _tmp1)) {
+				gee_list_set (((GeeList*) self->priv->base_types), i, new_type);
+				return;
+			}
+		}
+	}
+}
+
+
 ValaMethod* vala_struct_get_default_construction_method (ValaStruct* self) {
 	g_return_val_if_fail (VALA_IS_STRUCT (self), NULL);
 	return self->priv->_default_construction_method;
@@ -823,27 +883,28 @@ static void vala_struct_class_init (ValaStructClass * klass) {
 	G_OBJECT_CLASS (klass)->dispose = vala_struct_dispose;
 	VALA_CODE_NODE_CLASS (klass)->accept = vala_struct_real_accept;
 	VALA_CODE_NODE_CLASS (klass)->accept_children = vala_struct_real_accept_children;
-	VALA_DATA_TYPE_CLASS (klass)->get_cname = vala_struct_real_get_cname;
+	VALA_TYPESYMBOL_CLASS (klass)->get_cname = vala_struct_real_get_cname;
 	VALA_SYMBOL_CLASS (klass)->get_lower_case_cprefix = vala_struct_real_get_lower_case_cprefix;
 	VALA_SYMBOL_CLASS (klass)->get_lower_case_cname = vala_struct_real_get_lower_case_cname;
-	VALA_DATA_TYPE_CLASS (klass)->get_upper_case_cname = vala_struct_real_get_upper_case_cname;
-	VALA_DATA_TYPE_CLASS (klass)->get_type_id = vala_struct_real_get_type_id;
-	VALA_DATA_TYPE_CLASS (klass)->get_marshaller_type_name = vala_struct_real_get_marshaller_type_name;
-	VALA_DATA_TYPE_CLASS (klass)->get_get_value_function = vala_struct_real_get_get_value_function;
-	VALA_DATA_TYPE_CLASS (klass)->get_set_value_function = vala_struct_real_get_set_value_function;
-	VALA_DATA_TYPE_CLASS (klass)->get_default_value = vala_struct_real_get_default_value;
-	VALA_DATA_TYPE_CLASS (klass)->get_type_parameter_index = vala_struct_real_get_type_parameter_index;
+	VALA_TYPESYMBOL_CLASS (klass)->get_upper_case_cname = vala_struct_real_get_upper_case_cname;
+	VALA_TYPESYMBOL_CLASS (klass)->get_type_id = vala_struct_real_get_type_id;
+	VALA_TYPESYMBOL_CLASS (klass)->get_marshaller_type_name = vala_struct_real_get_marshaller_type_name;
+	VALA_TYPESYMBOL_CLASS (klass)->get_get_value_function = vala_struct_real_get_get_value_function;
+	VALA_TYPESYMBOL_CLASS (klass)->get_set_value_function = vala_struct_real_get_set_value_function;
+	VALA_TYPESYMBOL_CLASS (klass)->get_default_value = vala_struct_real_get_default_value;
+	VALA_TYPESYMBOL_CLASS (klass)->get_type_parameter_index = vala_struct_real_get_type_parameter_index;
+	VALA_CODE_NODE_CLASS (klass)->replace_type = vala_struct_real_replace_type;
 	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_STRUCT_DEFAULT_CONSTRUCTION_METHOD, g_param_spec_object ("default-construction-method", "default-construction-method", "default-construction-method", VALA_TYPE_METHOD, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
 }
 
 
 static void vala_struct_init (ValaStruct * self) {
 	self->priv = VALA_STRUCT_GET_PRIVATE (self);
-	self->priv->type_parameters = GEE_LIST (gee_array_list_new (VALA_TYPE_TYPE_PARAMETER, ((GBoxedCopyFunc) g_object_ref), g_object_unref, g_direct_equal));
+	self->priv->type_parameters = GEE_LIST (gee_array_list_new (VALA_TYPE_TYPEPARAMETER, ((GBoxedCopyFunc) g_object_ref), g_object_unref, g_direct_equal));
 	self->priv->constants = GEE_LIST (gee_array_list_new (VALA_TYPE_CONSTANT, ((GBoxedCopyFunc) g_object_ref), g_object_unref, g_direct_equal));
 	self->priv->fields = GEE_LIST (gee_array_list_new (VALA_TYPE_FIELD, ((GBoxedCopyFunc) g_object_ref), g_object_unref, g_direct_equal));
 	self->priv->methods = GEE_LIST (gee_array_list_new (VALA_TYPE_METHOD, ((GBoxedCopyFunc) g_object_ref), g_object_unref, g_direct_equal));
-	self->priv->base_types = GEE_LIST (gee_array_list_new (VALA_TYPE_TYPE_REFERENCE, ((GBoxedCopyFunc) g_object_ref), g_object_unref, g_direct_equal));
+	self->priv->base_types = GEE_LIST (gee_array_list_new (VALA_TYPE_DATA_TYPE, ((GBoxedCopyFunc) g_object_ref), g_object_unref, g_direct_equal));
 	self->priv->default_value = NULL;
 }
 
@@ -876,7 +937,7 @@ GType vala_struct_get_type (void) {
 	static GType vala_struct_type_id = 0;
 	if (G_UNLIKELY (vala_struct_type_id == 0)) {
 		static const GTypeInfo g_define_type_info = { sizeof (ValaStructClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_struct_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaStruct), 0, (GInstanceInitFunc) vala_struct_init };
-		vala_struct_type_id = g_type_register_static (VALA_TYPE_DATA_TYPE, "ValaStruct", &g_define_type_info, 0);
+		vala_struct_type_id = g_type_register_static (VALA_TYPE_TYPESYMBOL, "ValaStruct", &g_define_type_info, 0);
 	}
 	return vala_struct_type_id;
 }
