@@ -1,6 +1,6 @@
 /* valaunaryexpression.vala
  *
- * Copyright (C) 2006  Jürg Billeter
+ * Copyright (C) 2006-2007  Jürg Billeter
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -36,8 +36,9 @@ enum  {
 	VALA_UNARY_EXPRESSION_OPERATOR_STRING
 };
 static void vala_unary_expression_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor);
-static void vala_unary_expression_real_replace (ValaCodeNode* base, ValaCodeNode* old_node, ValaCodeNode* new_node);
+static void vala_unary_expression_real_replace_expression (ValaCodeNode* base, ValaExpression* old_node, ValaExpression* new_node);
 static char* vala_unary_expression_real_to_string (ValaCodeNode* base);
+static gboolean vala_unary_expression_real_is_pure (ValaExpression* base);
 static gpointer vala_unary_expression_parent_class = NULL;
 static void vala_unary_expression_dispose (GObject * obj);
 
@@ -54,7 +55,7 @@ ValaUnaryExpression* vala_unary_expression_new (ValaUnaryOperator op, ValaExpres
 	GParameter * __params;
 	GParameter * __params_it;
 	ValaUnaryExpression * self;
-	g_return_val_if_fail (VALA_IS_EXPRESSION (_inner), NULL);
+	g_return_val_if_fail (_inner == NULL || VALA_IS_EXPRESSION (_inner), NULL);
 	g_return_val_if_fail (source == NULL || VALA_IS_SOURCE_REFERENCE (source), NULL);
 	__params = g_new0 (GParameter, 1);
 	__params_it = __params;
@@ -74,19 +75,19 @@ ValaUnaryExpression* vala_unary_expression_new (ValaUnaryOperator op, ValaExpres
 static void vala_unary_expression_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor) {
 	ValaUnaryExpression * self;
 	self = VALA_UNARY_EXPRESSION (base);
-	g_return_if_fail (VALA_IS_CODE_VISITOR (visitor));
+	g_return_if_fail (visitor == NULL || VALA_IS_CODE_VISITOR (visitor));
 	vala_code_node_accept (VALA_CODE_NODE (vala_unary_expression_get_inner (self)), visitor);
 	vala_code_visitor_visit_unary_expression (visitor, self);
 }
 
 
-static void vala_unary_expression_real_replace (ValaCodeNode* base, ValaCodeNode* old_node, ValaCodeNode* new_node) {
+static void vala_unary_expression_real_replace_expression (ValaCodeNode* base, ValaExpression* old_node, ValaExpression* new_node) {
 	ValaUnaryExpression * self;
 	self = VALA_UNARY_EXPRESSION (base);
-	g_return_if_fail (VALA_IS_CODE_NODE (old_node));
-	g_return_if_fail (VALA_IS_CODE_NODE (new_node));
-	if (VALA_CODE_NODE (vala_unary_expression_get_inner (self)) == old_node) {
-		vala_unary_expression_set_inner (self, VALA_EXPRESSION (new_node));
+	g_return_if_fail (old_node == NULL || VALA_IS_EXPRESSION (old_node));
+	g_return_if_fail (new_node == NULL || VALA_IS_EXPRESSION (new_node));
+	if (vala_unary_expression_get_inner (self) == old_node) {
+		vala_unary_expression_set_inner (self, new_node);
 	}
 }
 
@@ -102,6 +103,16 @@ static char* vala_unary_expression_real_to_string (ValaCodeNode* base) {
 }
 
 
+static gboolean vala_unary_expression_real_is_pure (ValaExpression* base) {
+	ValaUnaryExpression * self;
+	self = VALA_UNARY_EXPRESSION (base);
+	if (vala_unary_expression_get_operator (self) == VALA_UNARY_OPERATOR_INCREMENT || vala_unary_expression_get_operator (self) == VALA_UNARY_OPERATOR_DECREMENT) {
+		return FALSE;
+	}
+	return vala_expression_is_pure (vala_unary_expression_get_inner (self));
+}
+
+
 ValaUnaryOperator vala_unary_expression_get_operator (ValaUnaryExpression* self) {
 	g_return_val_if_fail (VALA_IS_UNARY_EXPRESSION (self), 0);
 	return self->priv->_operator;
@@ -121,10 +132,12 @@ ValaExpression* vala_unary_expression_get_inner (ValaUnaryExpression* self) {
 
 
 void vala_unary_expression_set_inner (ValaUnaryExpression* self, ValaExpression* value) {
+	ValaExpression* _tmp2;
 	ValaExpression* _tmp1;
 	g_return_if_fail (VALA_IS_UNARY_EXPRESSION (self));
+	_tmp2 = NULL;
 	_tmp1 = NULL;
-	self->priv->_inner = (_tmp1 = g_object_ref (value), (self->priv->_inner == NULL ? NULL : (self->priv->_inner = (g_object_unref (self->priv->_inner), NULL))), _tmp1);
+	self->priv->_inner = (_tmp2 = (_tmp1 = value, (_tmp1 == NULL ? NULL : g_object_ref (_tmp1))), (self->priv->_inner == NULL ? NULL : (self->priv->_inner = (g_object_unref (self->priv->_inner), NULL))), _tmp2);
 	vala_code_node_set_parent_node (VALA_CODE_NODE (self->priv->_inner), VALA_CODE_NODE (self));
 }
 
@@ -206,8 +219,9 @@ static void vala_unary_expression_class_init (ValaUnaryExpressionClass * klass)
 	G_OBJECT_CLASS (klass)->set_property = vala_unary_expression_set_property;
 	G_OBJECT_CLASS (klass)->dispose = vala_unary_expression_dispose;
 	VALA_CODE_NODE_CLASS (klass)->accept = vala_unary_expression_real_accept;
-	VALA_CODE_NODE_CLASS (klass)->replace = vala_unary_expression_real_replace;
+	VALA_CODE_NODE_CLASS (klass)->replace_expression = vala_unary_expression_real_replace_expression;
 	VALA_CODE_NODE_CLASS (klass)->to_string = vala_unary_expression_real_to_string;
+	VALA_EXPRESSION_CLASS (klass)->is_pure = vala_unary_expression_real_is_pure;
 	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_UNARY_EXPRESSION_OPERATOR, g_param_spec_int ("operator", "operator", "operator", G_MININT, G_MAXINT, 0, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
 	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_UNARY_EXPRESSION_INNER, g_param_spec_object ("inner", "inner", "inner", VALA_TYPE_EXPRESSION, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT));
 	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_UNARY_EXPRESSION_OPERATOR_STRING, g_param_spec_string ("operator-string", "operator-string", "operator-string", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
